@@ -19,7 +19,7 @@ import os
 
 # application data
 TITLE = "Palabra"
-VERSION = "0.1.6"
+VERSION = "0.1.7"
 WEBSITE = "http://bitbucket.org/svisser/palabra"
 
 # configuration file
@@ -36,6 +36,11 @@ MISSING_CHAR = '?'
 WORDLIST_DIRECTORY = os.path.expanduser("~/.palabra/words")
 MAX_WORD_LENGTH = 64 # also in .c
 MAX_WORD_LISTS = 64 # also in .c
+MIN_WORD_SCORE = -1000000
+MAX_WORD_SCORE = 1000000
+
+# appearance
+MAX_LINE_WIDTH = 32
 
 # transform types (used in gui.py/update_window to indicate postprocessing)
 TRANSFORM_NONE = 0
@@ -57,6 +62,9 @@ FILL_OPTION_NICE = "nice"
 FILL_OPTION_DUPLICATE = "duplicate"
 FILL_NICE_COUNT = "nice_count"
 
+# search options, also in .c (search function)
+SEARCH_OPTION_MIN_SCORE = "min_score"
+
 # puzzle types
 PUZZLE_PALABRA = 'palabra'
 # http://www.xwordinfo.com/XPF/
@@ -71,6 +79,8 @@ SYM_90 = "90_degree"
 SYM_180 = "180_degree"
 SYM_DIAGONALS = "diagonals"
 
+DIRECTION_NAMES = {"across": "Across", "down": "Down"}
+
 # canonical identifiers for metadata across various file formats
 # (using simple Dublin Core)
 META_TITLE = 'title'
@@ -107,7 +117,7 @@ META_CODES = {
     , META_PUBLISHER: "%P"
     , META_DATE: "%D"
     , META_FILENAME: "%F"
-    , META_FILEPATH: "%FF"
+    , META_FILEPATH: "%L"
     , META_WIDTH: "%W"
     , META_HEIGHT: "%H"
     , META_N_WORDS: "%N"
@@ -115,6 +125,23 @@ META_CODES = {
     , META_PAGE_NUMBER: "%G"
 }
 
+# for header editor
+META_CAPTIONS = [
+    (META_CODES[META_TITLE], u"Title")
+    , (META_CODES[META_CREATOR], u"Author")
+    , (META_CODES[META_EDITOR], u"Editor")
+    , (META_CODES[META_COPYRIGHT], u"Rights")
+    , (META_CODES[META_PUBLISHER], u"Publisher")
+    , (META_CODES[META_DATE], u"Date")
+    , (META_CODES[META_FILENAME], u"File name")
+    , (META_CODES[META_FILEPATH], u"Path and file name")
+    , (META_CODES[META_WIDTH], u"Width")
+    , (META_CODES[META_HEIGHT], u"Height")
+    , (META_CODES[META_N_WORDS], "Number of words")
+    , (META_CODES[META_N_BLOCKS], "Number of blocks")
+    , (META_CODES[META_PAGE_NUMBER], "Page number")
+]
+
 # statusbar identifiers
 STATUS_MENU = "STATUS_MENU"
 STATUS_GRID = "STATUS_GRID"
@@ -145,7 +172,8 @@ VIEW_MODE_PREVIEW_SOLUTION = "preview_solution"
 WARN_UNCHECKED = "warn_unchecked_cells"
 WARN_CONSECUTIVE = "warn_consecutive_unchecked"
 WARN_TWO_LETTER = "warn_two_letter_words"
-WARNINGS = [WARN_UNCHECKED, WARN_CONSECUTIVE, WARN_TWO_LETTER]
+WARN_BLACKLIST = "warn_blacklist"
+WARNINGS = [WARN_UNCHECKED, WARN_CONSECUTIVE, WARN_TWO_LETTER, WARN_BLACKLIST]
 
 COLOR_WARNING = "color_warning"
 COLOR_HIGHLIGHT = "color_highlight"
@@ -156,6 +184,7 @@ COLOR_SECONDARY_ACTIVE = "color_secondary_active"
 
 INPUT_DELAY = 350
 INPUT_DELAY_SHORT = 250
+INPUT_DELAY_VERY_SHORT = 50
 
 PREF_COPY_BEFORE_SAVE = "backup_copy_before_save"
 PREF_INITIAL_HEIGHT = "new_initial_height"
@@ -163,4 +192,9 @@ PREF_INITIAL_WIDTH = "new_initial_width"
 PREF_UNDO_STACK_SIZE = "undo_stack_size"
 PREF_UNDO_FINITE_STACK = "undo_use_finite_stack"
 PREF_PATTERN_FILES = "pattern_files"
+PREF_CLUE_FILES = "clue_files"
 PREF_WORD_FILES = "word_files"
+PREF_ARROWS_CHANGE_DIR = "arrows_change_direction"
+PREF_BLACKLIST = "blacklist"
+PREF_FIND_WORD_FILES = "find_word_files"
+PREF_FIND_WORD_MIN_SCORE = "find_word_min_score"
