@@ -12,13 +12,34 @@ PyObject* find_matches(PyObject *list, Tptr p, char *s)
         if (p->splitchar && *s)
             find_matches(list, p->eqkid, s + 1);
     if (*s == 0 && p->splitchar == 0) {
-        PyList_Append(list, PyString_FromString(p->word));
+        PyObject *item = Py_BuildValue("(si)", p->word, p->score);
+        PyList_Append(list, item);
+        Py_DECREF(item);
+        // continue recursion as we may have more words
+        if (p->eqkid != NULL) {
+            find_matches(list, p->eqkid, s);
+        }
     }
     if (*s == '.' || *s > p->splitchar)
         find_matches(list, p->hikid, s);
     return list;
 }
 
+void update_score(Tptr p, char *s, int score)
+{
+    if (!p) return;
+    if (*s < p->splitchar)
+        update_score(p->lokid, s, score);
+    if (*s == p->splitchar)
+        if (p->splitchar && *s)
+            update_score(p->eqkid, s + 1, score);
+    if (*s == 0 && p->splitchar == 0) {
+        p->score = score;
+    }
+    if (*s > p->splitchar)
+        update_score(p->hikid, s, score);
+}
+
 void read_counts_str(char *counts_c, int *counts_i, char *s) {
     const int length = strlen(s);
     int i;
@@ -80,18 +101,14 @@ PyObject *find_matches_i(int index, char *s) {
     return result;
 }
 
-// 1 = ok, 0 = not ok
-int check_intersect(char *word, char **cs, int length, Sptr *results) {
+// fills in is_char_ok with 1 = ok, 0 = not ok
+void check_intersect(char *word, char **cs, int length, Sptr *results, int is_char_ok[MAX_WORD_LENGTH]) {
     int c;
-    for (c = 0; c < length; c++) {
-        if (results[c] == NULL || results[c]->n_matches == 0) {
-            return 0;
-        }
-    }
-    int n_chars = 0;
+    for (c = 0; c < length; c++)
+        if (results[c] == NULL)
+            return;
     for (c = 0; c < length; c++) {
         if (strchr(cs[c], '.') == NULL) {
-            n_chars += 1;
             continue;
         }
         int m;
@@ -99,12 +116,11 @@ int check_intersect(char *word, char **cs, int length, Sptr *results) {
             char m_c = results[c]->chars[m];
             if (m_c == ' ') break;
             if (m_c == *(word + c)) {
-                n_chars += 1;
+                is_char_ok[c] = 1;
                 break;
             }
         }
     }
-    return n_chars == length;
 }
 
 char* find_candidate(char **cs_i, Sptr *results, Slot *slot, char *cs, int option_nice, int offset) {
@@ -114,9 +130,31 @@ char* find_candidate(char **cs_i, Sptr *results, Slot *slot, char *cs, int optio
     Py_ssize_t m_count = 0;
     for (w = 0; w < count; w++) {
         char *word = PyString_AsString(PyList_GetItem(slot->words, w));
-        //printf("Considering %s %s\n", word, cs);
-        if (check_constraints(word, cs) && (option_nice ? 1 : check_intersect(word, cs_i, slot->length, results))) {
-            //printf("checking %s\n", word);
+        if (check_constraints(word, cs)) {
+            //printf("Considering %i %s %s\n", option_nice, word, cs);
+            if (!option_nice) {
+                // TODO refactor with cPalabra_search
+                int is_char_ok[MAX_WORD_LENGTH];
+                int j = 0;
+                for (j = 0; j < slot->length; j++) {
+                    is_char_ok[j] = 0;
+                }
+                check_intersect(word, cs_i, slot->length, results, is_char_ok);
+                int n_chars = 0;
+                // mark fully filled in intersecting words also as ok
+                for (j = 0; j < slot->length; j++) {
+                    if (strchr(cs_i[j], '.') == NULL) {
+                        n_chars++;
+                    }
+                }
+                for (j = 0; j < slot->length; j++) {
+                    if (is_char_ok[j]) n_chars++;
+                }
+                //printf("%i %i\n", n_chars, slot->length);
+                if (n_chars < slot->length)
+                    continue;
+            }
+            //printf("checking %i %i\n", m_count, offset);
             if (m_count == offset) {
                 return word;
             }
@@ -200,36 +238,40 @@ void print(Tptr p, int indent)
     if (p->hikid != NULL) print(p->hikid, indent + 2);
 }
 
-Tptr insert1(Tptr p, char *s, char *word)
+Tptr insert1(Tptr p, char *s, char *word, int score)
 {
     if (p == NULL) {
         p = (Tptr) PyMem_Malloc(sizeof(Tnode));
         p->splitchar = *s;
         p->word = word;
+        p->score = score;
         p->lokid = NULL;
         p->eqkid = NULL;
         p->hikid = NULL;
+    } else if (*s == 0 && p != NULL) {
+        // if we are inserting the same word more than once, continue recursion
+        p->eqkid = insert1(p->eqkid, s, word, score);
     }
     if (*s < p->splitchar)
-        p->lokid = insert1(p->lokid, s, word);
+        p->lokid = insert1(p->lokid, s, word, score);
     else if (*s == p->splitchar) {
         if (*s != 0)
-            p->eqkid = insert1(p->eqkid, ++s, word);
+            p->eqkid = insert1(p->eqkid, ++s, word, score);
     } else
-        p->hikid = insert1(p->hikid, s, word);
+        p->hikid = insert1(p->hikid, s, word, score);
     return p;
 }
 
-int analyze(int offset, Sptr result, Tptr p, char *s, char *cs)
+int analyze(int offset, Sptr result, Tptr p, char *s, char *cs, int min_score)
 {
     if (!p) return 0;
     int n = 0;
     if (*s == '.' || *s < p->splitchar)
-        n += analyze(offset, result, p->lokid, s, cs);
+        n += analyze(offset, result, p->lokid, s, cs, min_score);
     if (*s == '.' || *s == p->splitchar)
         if (p->splitchar && *s)
-            n += analyze(offset, result, p->eqkid, s + 1, cs);
-    if (*s == 0 && p->splitchar == 0) {
+            n += analyze(offset, result, p->eqkid, s + 1, cs, min_score);
+    if (*s == 0 && p->splitchar == 0 && p->score >= min_score) {
         n += 1;
         char intersect_char = *(cs + offset);
         if (intersect_char == '.') {
@@ -248,13 +290,13 @@ int analyze(int offset, Sptr result, Tptr p, char *s, char *cs)
         }
     }
     if (*s == '.' || *s > p->splitchar)
-        n += analyze(offset, result, p->hikid, s, cs);
+        n += analyze(offset, result, p->hikid, s, cs, min_score);
     result->n_matches = n;
     return n;
 }
 
-Sptr analyze_intersect_slot(int offset, char *cs) {
-    if (!trees[0][strlen(cs)]) {
+Sptr analyze_intersect_slot(int offset, char *cs, int index, int min_score) {
+    if (!trees[index][strlen(cs)]) {
         return NULL;
     }
     Sptr result;
@@ -271,11 +313,11 @@ Sptr analyze_intersect_slot(int offset, char *cs) {
     for (c = 0; c < MAX_ALPHABET_SIZE; c++) {
         result->chars[c] = ' ';
     }
-    analyze(offset, result, trees[0][strlen(cs)], cs, cs);
+    analyze(offset, result, trees[index][strlen(cs)], cs, cs, min_score);
     return result;
 }
 
-void analyze_intersect_slot2(Sptr *results, int *skipped, int *offsets, char **cs, int length) {
+void analyze_intersect_slot2(Sptr *results, int *skipped, int *offsets, char **cs, int length, int index, int min_score) {
     int t;
     for (t = 0; t < length; t++) {
         int skip = -1;
@@ -289,7 +331,7 @@ void analyze_intersect_slot2(Sptr *results, int *skipped, int *offsets, char **c
             }
         }
         if (skip < 0) {
-            results[t] = analyze_intersect_slot(offsets[t], cs[t]);
+            results[t] = analyze_intersect_slot(offsets[t], cs[t], index, min_score);
         } else {
             skipped[t] = 1;
             results[t] = results[skip];
@@ -384,7 +426,12 @@ int count_words(PyObject *words, int length, char *cs) {
     PyObject* key = Py_BuildValue("i", length);
     PyObject* words_m = PyDict_GetItem(words, key);
     for (w = 0; w < PyList_Size(words_m); w++) {
-        char *word = PyString_AsString(PyList_GetItem(words_m, w));
+        PyObject* word_obj = PyList_GET_ITEM(words_m, w);
+        PyObject* word_str;
+        const int word_score;
+        if (!PyArg_ParseTuple(word_obj, "Oi", &word_str, &word_score))
+            return -1;
+        char *word = PyString_AsString(word_str);
         if (!check_constraints(word, cs)) {
             continue;
         }
@@ -433,7 +480,11 @@ void clear_slot(Cell *cgrid, int width, int height, Slot *slots, int n_slots, in
         int m = get_slot_index(slots, n_slots, cx, cy, slot.dir == DIR_ACROSS ? 1 : 0);
         int is_inter_ok = 1;
         if (m >= 0) {
-            is_inter_ok = 1; // TODO can_clear_char(cgrid, width, height, slots[m]);
+            // a char can be cleared if the intersecting slot is not done or
+            // when it has one or more missing characters (these are not mutually
+            // exclusive as a slot can be fully filled in but not yet marked as done:
+            // the content of the slot may not be a valid word)
+            is_inter_ok = !slots[m].done || can_clear_char(cgrid, width, height, slots[m]);
         }
         Cell *cell = &cgrid[cx + cy * width];
         if (is_inter_ok && cell->c != CONSTRAINT_EMPTY) {
@@ -473,38 +524,34 @@ char* get_constraints(Cell *cgrid, int width, int height, Slot *slot) {
     if (!cs) {
         return NULL;
     }
-    int dx = slot->dir == DIR_ACROSS ? 1 : 0;
-    int dy = slot->dir == DIR_DOWN ? 1 : 0;
-    int x = slot->x;
-    int y = slot->y;
-    int count = 0;
-    while (is_available(cgrid, width, height, x, y)) {
-        //printf("Setting %i for %i %i to %c\n", count, x, y, cgrid[x + y * height].c);
-        cs[count] = cgrid[x + y * width].c;
-        if (dx == 1 && is_valid(x + dx, y, width, height) && cgrid[(x + dx) + y * width].left_bar == 1)
-            break;
-        if (dy == 1 && is_valid(x, y + dy, width, height) && cgrid[x + (y + dy) * width].top_bar == 1)
-            break;
-        x += dx;
-        y += dy;
-        count++;
+    get_constraints_i(cgrid, width, height, slot, cs);
+    return cs;
+}
+
+void get_constraints_i(Cell *cgrid, int width, int height, Slot *slot, char *cs) {
+    const int dx = slot->dir == DIR_ACROSS ? 1 : 0;
+    const int dy = slot->dir == DIR_DOWN ? 1 : 0;
+    int j;
+    for (j = 0; j < slot->length; j++) {
+        cs[j] = cgrid[(slot->x + j * dx) + (slot->y + j * dy) * width].c;
     }
     cs[slot->length] = '\0';
-    return cs;
 }
 
 int determine_count(PyObject *words, Cell *cgrid, int width, int height, Slot *slot) {
     int prev = slot->count;
-    char *ds = get_constraints(cgrid, width, height, slot);
-    if (!ds) {
-        printf("Warning: determine_count failed to obtain constraints.\n");
+    // TODO reduce these malloc calls
+    //printf("Constraint search\n");
+    char* cs = PyMem_Malloc(slot->length * sizeof(char) + 1);
+    if (!cs) {
         return -1;
     }
-    int count = count_words(words, slot->length, ds);
+    get_constraints_i(cgrid, width, height, slot, cs);
+    int count = count_words(words, slot->length, cs);
     if (DEBUG && count == 0) {
         printf("WARNING: slot (%i, %i, %i): from %i to %i\n", slot->x, slot->y, slot->dir, prev, count);
     }
-    PyMem_Free(ds);
+    PyMem_Free(cs);
     return count;
 }
 
@@ -541,7 +588,7 @@ int backtrack(PyObject *words, Cell *cgrid, int width, int height, Slot *slots,
                 continue;
             }
             Slot *bslot = &slots[blank];
-            if (0) printf("Removing: (%i, %i, %s)\n", bslot->x, bslot->y, bslot->dir == DIR_ACROSS ? "across" : "down");
+            //printf("About to clear: (%i, %i, %s)\n", bslot->x, bslot->y, bslot->dir == DIR_ACROSS ? "across" : "down");
             cleared++;
             clear_slot(cgrid, width, height, slots, n_slots, blank);
             bslot->count = determine_count(words, cgrid, width, height, bslot);
@@ -555,21 +602,6 @@ int backtrack(PyObject *words, Cell *cgrid, int width, int height, Slot *slots,
                     iindex -= 1;
                 }
             }
-            // reset offsets of intersecting slots as well
-            // the reason is that constraints may change with future words
-            // for the blanked slot which means offsets have to be reset
-            // TODO: improve as we may be throwing away too much info
-            int k;
-            for (k = 0; k < bslot->length; k++) {
-                int cx = bslot->x + (bslot->dir == DIR_ACROSS ? k : 0);
-                int cy = bslot->y + (bslot->dir == DIR_DOWN ? k : 0);
-                int dir = bslot->dir == DIR_ACROSS ? 1 : 0;
-                int indexD = get_slot_index(slots, n_slots, cx, cy, dir);
-                if (indexD >= 0 && indexD != blank) {
-                    (&slots[indexD])->offset = 0;
-                }
-            }
-            
         }
     }
     return cleared;
