@@ -25,19 +25,27 @@ cPalabra_search(PyObject *self, PyObject *args) {
     PyObject *constraints;
     PyObject *more_constraints;
     PyObject *indices;
-    if (!PyArg_ParseTuple(args, "iOOO", &length, &constraints, &more_constraints, &indices))
+    PyObject *options;
+    if (!PyArg_ParseTuple(args, "iOOOO", &length, &constraints, &more_constraints, &indices, &options))
         return NULL;
     if (length <= 0 || length >= MAX_WORD_LENGTH)
         return PyList_New(0);
     char *cons_str = PyString_AS_STRING(constraints);
+    const Py_ssize_t n_indices = PyList_Size(indices);
     
+    const int HAS_OPTIONS = options != Py_None;
+    int OPTION_MIN_SCORE = -9999; // TODO ugly
+    if (HAS_OPTIONS) {
+        OPTION_MIN_SCORE = (int) PyInt_AsLong(PyDict_GetItem(options, PyString_FromString("min_score")));
+    }
+
     // each of the constraints
     int offsets[length];
     char *cs[length];
     int t;
     int skipped[length];
     for (t = 0; t < length; t++) skipped[t] = 0;
-    Sptr results[length];
+    Sptr results[n_indices][length];
     if (more_constraints != Py_None) {
         for (t = 0; t < length; t++) {
             PyObject *py_cons_str2;
@@ -46,36 +54,71 @@ cPalabra_search(PyObject *self, PyObject *args) {
                 return NULL;
             cs[t] = PyString_AS_STRING(py_cons_str2);
         }
-        analyze_intersect_slot2(results, skipped, offsets, cs, length);
+        Py_ssize_t ii;
+        for (ii = 0; ii < n_indices; ii++) {
+            const int index = (int) PyInt_AsLong(PyList_GET_ITEM(indices, ii));
+            analyze_intersect_slot2(results[ii], skipped, offsets, cs, length, index, OPTION_MIN_SCORE);
+        }
     }
 
     // main word
     PyObject *result = PyList_New(0);
     Py_ssize_t ii;
-    for (ii = 0; ii < PyList_Size(indices); ii++) {
+    for (ii = 0; ii < n_indices; ii++) {
         const int index = (int) PyInt_AsLong(PyList_GET_ITEM(indices, ii));
-        
         PyObject *mwords = PyList_New(0);
         mwords = find_matches(mwords, trees[index][strlen(cons_str)], cons_str);
         Py_ssize_t m;
         for (m = 0; m < PyList_Size(mwords); m++) {
-            char *word = PyString_AS_STRING(PyList_GET_ITEM(mwords, m));
+            PyObject* m_item = PyList_GET_ITEM(mwords, m);
+            PyObject* word_str;
+            const int score;
+            if (!PyArg_ParseTuple(m_item, "Oi", &word_str, &score))
+                return NULL;
+            if (HAS_OPTIONS && score < OPTION_MIN_SCORE)
+                continue;
+            char *word = PyString_AS_STRING(word_str);
             int valid = 1;
             if (more_constraints != Py_None) {
-                valid = check_intersect(word, cs, length, results);
+                valid = 0;
+                // TODO refactor with find_candidate
+                int is_char_ok[MAX_WORD_LENGTH];
+                int i;
+                for (i = 0; i < length; i++) {
+                    is_char_ok[i] = 0;
+                }
+                // mark fully filled in intersecting words also as ok
+                for (i = 0; i < length; i++) {
+                    if (strchr(cs[i], '.') == NULL)
+                        is_char_ok[i] = 1;
+                }
+                Py_ssize_t jj;
+                for (jj = 0; jj < n_indices; jj++) {
+                    check_intersect(word, cs, length, results[jj], is_char_ok);
+                    int n_chars = 0;
+                    int j;
+                    for (j = 0; j < length; j++) {
+                        if (is_char_ok[j]) n_chars++;
+                    }
+                    if (n_chars == length) {
+                        valid = 1;
+                        break;
+                    }
+                }
             }
-            PyObject* py_intersect = PyBool_FromLong(valid);
-            PyObject* item = Py_BuildValue("(sO)", word, py_intersect);
-            Py_DECREF(py_intersect);
+            PyObject* item = Py_BuildValue("(sib)", word, score, valid);
             PyList_Append(result, item);
             Py_DECREF(item);
         }
-    }    
+    }
     if (more_constraints != Py_None) {
-        for (t = 0; t < length; t++) {
-            if (skipped[t] == 0 && results[t] != NULL) {
-                PyMem_Free(results[t]->chars);
-                PyMem_Free(results[t]);
+        Py_ssize_t ii;
+        for (ii = 0; ii < n_indices; ii++) {
+            for (t = 0; t < length; t++) {
+                if (skipped[t] == 0 && results[ii][t] != NULL) {
+                    PyMem_Free(results[ii][t]->chars);
+                    PyMem_Free(results[ii][t]);
+                }
             }
         }
     }
@@ -102,7 +145,10 @@ cPalabra_preprocess(PyObject *self, PyObject *args) {
     if (!PyArg_ParseTuple(args, "Oi", &words, &index))
         return NULL;
     
-    // create dict (keys are word lengths, each item is a list with words of that length)
+    // create dict
+    // keys = word lengths
+    // values = list with words of that length
+    // each item has (word, score)
     PyObject* dict = PyDict_New();
     PyObject* keys[MAX_WORD_LENGTH];
     int l;
@@ -117,34 +163,56 @@ cPalabra_preprocess(PyObject *self, PyObject *args) {
     for (w = 0; w < PyList_Size(words); w++) {
         PyObject* word = PyList_GET_ITEM(words, w);
         PyObject* word_str;
-        const int word_rank;
-        if (!PyArg_ParseTuple(word, "Oi", &word_str, &word_rank))
+        const int word_score;
+        if (!PyArg_ParseTuple(word, "Oi", &word_str, &word_score))
             return NULL;
         int length = (int) PyString_GET_SIZE(word_str);
         if (length <= 0 || length >= MAX_WORD_LENGTH)
             continue;
         PyObject* key = keys[length];
         // PyDict_GetItem eats ref
-        PyList_Append(PyDict_GetItem(dict, key), word_str);
+        PyList_Append(PyDict_GetItem(dict, key), word);
     }
 
     // build ternary search trees per word length
     // TODO insert in random order for best performance
     int m;
     for (m = 0; m < MAX_WORD_LENGTH; m++) {
+        // clear previous tree in case we are rebuilding for this index
+        if (trees[index][m] != NULL) {
+            free_tree(trees[index][m]);
+            PyMem_Free(trees[index][m]);
+        }
         trees[index][m] = NULL;
         PyObject *key = Py_BuildValue("i", m);
         PyObject *words = PyDict_GetItem(dict, key);
         const Py_ssize_t len_m = PyList_Size(words);
         Py_ssize_t w;
         for (w = 0; w < len_m; w++) {
-            char *word = PyString_AsString(PyList_GET_ITEM(words, w));
-            trees[index][m] = insert1(trees[index][m], word, word);
+            PyObject *w_word = PyList_GET_ITEM(words, w);
+            PyObject* w_str;
+            const int w_score;
+            if (!PyArg_ParseTuple(w_word, "Oi", &w_str, &w_score))
+                return NULL;
+            char *c_word = PyString_AsString(w_str);
+            trees[index][m] = insert1(trees[index][m], c_word, c_word, w_score);
         }
     }
     return dict;
 }
 
+static PyObject*
+cPalabra_insert_word(PyObject *self, PyObject *args) {
+    const int index;
+    const int length;
+    char *word;
+    const int score;
+    if (!PyArg_ParseTuple(args, "iisi", &index, &length, &word, &score))
+        return NULL;
+    trees[index][length] = insert1(trees[index][length], word, word, score);
+    return Py_None;
+}
+
 static PyObject*
 cPalabra_postprocess(PyObject *self, PyObject *args) {
     int i;
@@ -235,6 +303,8 @@ cPalabra_fill(PyObject *self, PyObject *args) {
     const int height = (int) PyInt_AsLong(PyObject_GetAttrString(grid, "height"));
     PyObject* data = PyObject_GetAttrString(grid, "data");
     
+    const int OUTPUT_DEBUG = 0;
+    
     int x;
     int y;
     Cell cgrid[width * height];
@@ -294,24 +364,24 @@ cPalabra_fill(PyObject *self, PyObject *args) {
             cgrid[index].fixed = 1;
         }
         
-        char *cs = get_constraints(cgrid, width, height, &slots[m]);
-        if (!cs) {
+        slots[m].cs = PyMem_Malloc(length * sizeof(char) + 1);
+        if (!slots[m].cs) {
             printf("Warning: fill failed to obtain constraints.\n");
             return NULL;
         }
-        slots[m].count = count_words(words, length, cs);
+        get_constraints_i(cgrid, width, height, &slots[m], slots[m].cs);
+        slots[m].count = count_words(words, length, slots[m].cs);
         slots[m].done = 1;
         slots[m].offset = 0;
         int j;
         for (j = 0; j < length; j++) {
-            if (cs[j] == CONSTRAINT_EMPTY) {
+            if (slots[m].cs[j] == CONSTRAINT_EMPTY) {
                 slots[m].done = 0;
             }
         }
         if (slots[m].done) {
             n_done_slots++;
         }
-        PyMem_Free(cs);
     }
     
     int order[n_slots];
@@ -324,7 +394,7 @@ cPalabra_fill(PyObject *self, PyObject *args) {
     PyObject *result = PyList_New(0);
     PyObject *best_fill = NULL;
     int best_n_done_slots = 0;
-    while (attempts < 100) {
+    while (attempts < 1000) {
         int index = -1;
         if (OPTION_NICE) {
             index = find_nice_slot(words, slots, n_slots, width, height, order);
@@ -333,18 +403,12 @@ cPalabra_fill(PyObject *self, PyObject *args) {
         } else {
             index = find_slot(slots, n_slots, order);
         }
-        //printf("Index: %i\n", index);
         if (index < 0) break;
         Slot *slot = &slots[index];
         if (DEBUG) {
             printf("Searching word for (%i, %i, %s, %i) at index %i: \n", slot->x, slot->y, slot->dir == 0 ? "across" : "down", slot->count, index);
         }
-        char *cs = get_constraints(cgrid, width, height, slot);
-        //printf("Obtained %s\n", cs);
-        if (!cs) {
-            printf("Warning: fill failed to obtain constraints.\n");
-            return NULL;
-        }
+        get_constraints_i(cgrid, width, height, slot, slot->cs);
         
         char *cs_i[slot->length];
         for (m = 0; m < slot->length; m++) {
@@ -370,11 +434,14 @@ cPalabra_fill(PyObject *self, PyObject *args) {
                         index = (&slots[m])->y - slot->y;
                         offset = slot->x - (&slots[m])->x;
                     }
+                    cs_i[index] = PyMem_Malloc(slots[m].length * sizeof(char) + 1);
                     offsets[index] = offset;
-                    cs_i[index] = get_constraints(cgrid, width, height, &slots[m]);
+                    get_constraints_i(cgrid, width, height, &slots[m], cs_i[index]);
                 }
             }
-            analyze_intersect_slot2(results, skipped, offsets, cs_i, slot->length);
+            // TODO index
+            // TODO min_score
+            analyze_intersect_slot2(results, skipped, offsets, cs_i, slot->length, 0, -9999);
         }
         
         int is_word_ok = 1;
@@ -385,20 +452,31 @@ cPalabra_fill(PyObject *self, PyObject *args) {
         }*/
         
         //printf("Trying for %i %i %i\n", slot->x, slot->y, slot->dir);
-        char* word = find_candidate(cs_i, results, slot, cs, OPTION_NICE, slot->offset);
-        //printf("Candidate BEFORE: %s (%i %i %i) from %i\n", word, slot->x, slot->y, slot->dir, slot->offset);
+        char* word = find_candidate(cs_i, results, slot, slot->cs, OPTION_NICE, slot->offset);
+        //if (word) printf("before %s at %i %i %i from %i\n", word, slot->x, slot->y, slot->dir, slot->offset);
+        if (0) {
+            PyObject *val = Py_BuildValue("(ssiiii)", "before", word, slot->x, slot->y, slot->dir, slot->offset);
+            PyList_Append(result, val);
+            Py_DECREF(val);
+        }
         
         if (word && OPTION_DUPLICATE) {
+            int duplicates[n_slots];
+            for (t = 0; t < n_slots; t++) {
+                duplicates[t] = 0;
+            }
             while (1) {
-                int next_attempts = 0;
+                int added_offset = 0;
                 int next = 0;
                 for (t = 0; t < n_slots; t++) {
                     if (!slots[t].done) continue;
+                    if (duplicates[t]) added_offset++;
                     char *word_t = get_constraints(cgrid, width, height, &slots[t]);
                     if (word_t && strcmp(word, word_t) == 0) {
-                        next_attempts++;
-                        //printf("Finding candidate for %i %i %i with offset %i\n", slot->x, slot->y, slot->dir, slot->offset + next_attempts);
-                        word = find_candidate(cs_i, results, slot, cs, OPTION_NICE, slot->offset + next_attempts);
+                        PyMem_Free(word_t);
+                        duplicates[t] = 1;
+                        added_offset++;
+                        word = find_candidate(cs_i, results, slot, slot->cs, OPTION_NICE, slot->offset + added_offset);
                         next = word != NULL;
                         break;
                     }
@@ -406,11 +484,13 @@ cPalabra_fill(PyObject *self, PyObject *args) {
                 if (!next) break;
             }
         }
-        //if (word) printf("Candidate AFTER: %s (%i %i %i)\n", word, slot->x, slot->y, slot->dir);
-        
-        if (cs != NULL) {
-            PyMem_Free(cs);
+        //if (word) printf("after %s at %i %i %i from %i\n", word, slot->x, slot->y, slot->dir, slot->offset);
+        if (0 && word) {
+            PyObject *val = Py_BuildValue("(ssiiii)", "after", word, slot->x, slot->y, slot->dir, slot->offset);
+            PyList_Append(result, val);
+            Py_DECREF(val);
         }
+        
         for (m = 0; m < slot->length; m++) {
             if (cs_i[m] != NULL) {
                 PyMem_Free(cs_i[m]);
@@ -468,6 +548,11 @@ cPalabra_fill(PyObject *self, PyObject *args) {
                     if (affected[k] >= 0) {
                         int count = determine_count(words, cgrid, width, height, &slots[affected[k]]);
                         (&slots[affected[k]])->count = count;
+                        // if an intersecting slot is not yet done, reset
+                        // offset because constraints have changed.
+                        if (!slots[affected[k]].done) {
+                            (&slots[affected[k]])->offset = 0;
+                        }
                     }
                 }
             }
@@ -493,13 +578,18 @@ cPalabra_fill(PyObject *self, PyObject *args) {
                 n_done_slots -= cleared;
             }
         }
+        if (OUTPUT_DEBUG) {
+            PyObject *val = gather_fill(cgrid, width, height);
+            PyList_Append(result, val);
+            Py_DECREF(val);
+        }
         if (is_word_ok) {
             slot->done = 1;
             order[n_done_slots] = index;
             n_done_slots++;
             //printf("Filled in: %s (%i %i %i)\n", word, slot->x, slot->y, slot->dir);
         }
-        if (n_done_slots == NICE_COUNT) break;
+        if (OPTION_NICE && n_done_slots == NICE_COUNT) break;
         attempts++;
     }
     if (best_fill == NULL && (OPTION_NICE ? n_done_slots == NICE_COUNT : 1)) {
@@ -509,11 +599,15 @@ cPalabra_fill(PyObject *self, PyObject *args) {
         PyList_Append(result, best_fill);
         Py_DECREF(best_fill);
     }
+    for (m = 0; m < n_slots; m++) {
+        PyMem_Free(slots[m].cs);
+    }
     return result;
 }
 
 static PyObject*
 cPalabra_compute_lines(PyObject *self, PyObject *args) {
+    // left lines must be returned before top lines (needed for compute_render_lines)
     PyObject *grid;
     if (!PyArg_ParseTuple(args, "O", &grid))
         return NULL;
@@ -549,6 +643,7 @@ cPalabra_compute_lines(PyObject *self, PyObject *args) {
             Py_DECREF(item);
             Py_DECREF(py_void);
             
+            // e == 0 (left) or e == 1 (top)
             for (e = 0; e < 2; e++) {
                 int dx = e == 0 ? -1 : 0;
                 int dy = e == 0 ? 0 : -1;
@@ -688,6 +783,21 @@ cPalabra_compute_render_lines(PyObject *self, PyObject *args) {
         if (!PyArg_ParseTuple(cell, "ii", &x, &y))
             return NULL;
         PyObject *cell_lines = PyDict_GetItem(grid_lines, cell);
+        
+        int has_left_border_line = 0;
+        int has_right_border_line = 0;
+        
+        // peek at cell to the right to see if there's a border
+        Py_ssize_t m;
+        for (m = 0; m < n_lines; m++) {
+            if (all_x[m] == x + 1
+                && all_y[m] == y
+                && strcmp(all_t[m], "left") == 0
+                && strcmp(all_s[m], "innerborder") == 0) {
+                has_right_border_line = 1;
+                break;
+            }
+        }
 
         Py_ssize_t l;
         for (l = 0; l < PyList_Size(cell_lines); l++) {
@@ -754,55 +864,105 @@ cPalabra_compute_render_lines(PyObject *self, PyObject *args) {
                 }
             }
             if (l_is_left) {
-                //if ((x == 1 && y == 0) || (x == 1 && y == 1))
-                //    printf("%i %i | %f %f %f %f | %f %f\n", x, y, sx_p, sy_q, start, cell_size, sx_p + start, sy_q);
+                has_left_border_line = l_is_border;
                 PyObject* r = Py_BuildValue("(ffifiii)", sx_p + start, sy_q, 0, cell_size, bar, l_is_border, 0);
                 PyList_Append(result, r);
                 Py_DECREF(r);
             } else if (l_is_top) {
                 int is_lb = 1;
                 int is_rb = 1;
-                int dxl = 0;
-                int dxr = 0;
-
+                // TODO refactor
                 int x1_y1_left_outerborder = 0;
                 int x1_y_left_outerborder = 0;
                 int x_y1_left_innerborder = 0;
                 int x_y_left_innerborder = 0;
+                int x_y_left_outerborder = 0;
+                int xm1_y_top_innerborder = 0;
+                int x_ym1_left_outerborder = 0;
+                int xm1_y_top_outerborder = 0;
+                int x1_y_left_innerborder = 0;
+                int x1_y_top_innerborder = 0;
+                int x1_y_top_outerborder = 0;
+                int x1_ym1_left_innerborder = 0;
+                int x1_ym1_left_outerborder = 0;
+                int x_ym1_left_innerborder = 0;
                 Py_ssize_t v;
                 for (v = 0; v < n_lines; v++) {
-                    if (strcmp(all_t[v], "left") != 0) continue;
                     const int v_x = all_x[v];
                     const int v_y = all_y[v];
                     const int is_outerborder = strcmp(all_s[v], "outerborder") == 0;
                     const int is_innerborder = strcmp(all_s[v], "innerborder") == 0;
-                    if (v_x == x + 1 && v_y == y - 1 && is_outerborder)
+                    const int is_left = strcmp(all_t[v], "left") == 0;
+                    if (v_x == x + 1 && v_y == y - 1 && is_outerborder && is_left)
                         x1_y1_left_outerborder = 1;
-                    if (v_x == x + 1 && v_y == y && is_outerborder)
+                    if (v_x == x + 1 && v_y == y && is_outerborder && is_left)
                         x1_y_left_outerborder = 1;
-                    if (v_x == x && v_y == y - 1 && is_innerborder)
+                    if (v_x == x && v_y == y - 1 && is_innerborder && is_left)
                         x_y1_left_innerborder = 1;
-                    if (v_x == x && v_y == y && is_innerborder)
+                    if (v_x == x && v_y == y && is_innerborder && is_left)
                         x_y_left_innerborder = 1;
+                    if (v_x == x && v_y == y && is_outerborder && is_left)
+                        x_y_left_outerborder = 1;
+                    if (v_x == x - 1 && v_y == y && is_innerborder && !is_left)
+                        xm1_y_top_innerborder = 1;
+                    if (v_x == x - 1 && v_y == y && is_outerborder && !is_left)
+                        xm1_y_top_outerborder = 1;
+                    if (v_x == x && v_y == y - 1 && is_outerborder && is_left)
+                        x_ym1_left_outerborder = 1;
+                    if (v_x == x + 1 && v_y == y && is_innerborder && is_left)
+                        x1_y_left_innerborder = 1;
+                    if (v_x == x + 1 && v_y == y && is_innerborder && !is_left)
+                        x1_y_top_innerborder = 1;
+                    if (v_x == x + 1 && v_y == y && is_outerborder && !is_left)
+                        x1_y_top_outerborder = 1;
+                    if (v_x == x + 1 && v_y == y - 1 && is_innerborder && is_left)
+                        x1_ym1_left_innerborder = 1;
+                    if (v_x == x + 1 && v_y == y - 1 && is_outerborder && is_left)
+                        x1_ym1_left_outerborder = 1;
+                    if (v_x == x && v_y == y - 1 && is_innerborder && is_left)
+                        x_ym1_left_innerborder = 1;
                 }
+                // exclude cases in which lb or rb is not needed
                 if (x1_y1_left_outerborder || x1_y_left_outerborder) is_rb = 0;
                 if (x_y1_left_innerborder || x_y_left_innerborder) is_lb = 0;
                 
-                float rx = sx_p - dxl;
+                // corners on left side
+                if (xm1_y_top_innerborder && x_y_left_outerborder) is_lb = 0;
+                if (xm1_y_top_outerborder && x_ym1_left_outerborder) is_lb = 0;
+                
+                // corners on right side
+                if (x1_y_left_innerborder && x1_y_top_innerborder) is_rb = 0;
+                if (x1_ym1_left_innerborder && x1_y_top_outerborder) is_rb = 0;
+                
+                float rx = sx_p;
                 float ry = sy_q + start;
-                float rdx = cell_size + dxl + dxr;
+                float rdx = cell_size;
+                // extend borders to make them fit in the corners
+                if (l_is_border) {
+                    if (x_y_left_innerborder || x_ym1_left_innerborder) {
+                        rx -= line_width;
+                        rdx += line_width;
+                    }
+                    if (x1_ym1_left_outerborder || x1_y_left_outerborder) {
+                        rdx += line_width;
+                    }
+                }
                 PyObject* r = Py_BuildValue("(fffiiii)", rx, ry, rdx, 0, bar, l_is_border, 0);
                 PyList_Append(result, r);
                 Py_DECREF(r);
+                // lines that are sticking out (could be normal lines or borders)
+                // these are marked as 'special'
                 if (is_lb) {
+                    int is_border = l_is_border || has_left_border_line;
                     PyObject *r1 = Py_BuildValue("(ffiiiii)"
-                        , sx_p - dxl - border_width, sy_q + start, border_width, 0, 0, l_is_border, 1);
+                        , sx_p - border_width, sy_q + start, border_width, 0, 0, is_border, 1);
                     PyList_Append(result, r1);
                     Py_DECREF(r1);
                 }
                 if (is_rb) {
+                    int is_border = l_is_border || has_right_border_line;
                     PyObject *r2 = Py_BuildValue("(ffiiiii)"
-                        , sx_p + cell_size, sy_q + start, border_width, 0, 0, l_is_border, 1);
+                        , sx_p + cell_size, sy_q + start, border_width, 0, 0, is_border, 1);
                     PyList_Append(result, r2);
                     Py_DECREF(r2);
                 }
@@ -814,7 +974,7 @@ cPalabra_compute_render_lines(PyObject *self, PyObject *args) {
 
 static PyObject*
 cPalabra_compute_distances(PyObject *self, PyObject *args) {
-    /*
+    /* NO LONGER UPDATED:
     def compute_distance(w):
         places = 0
         for i, c in enumerate(w):
@@ -858,6 +1018,7 @@ cPalabra_compute_distances(PyObject *self, PyObject *args) {
             PyObject *a_l = PyDict_GetItem(counts, py_lengths[l]);
             PyObject *a_l_i = PyDict_GetItem(a_l, py_lengths[l_i]);
             
+            int occurs = 0;
             int j;
             for (j = 0; j < PyList_GET_SIZE(a_l_i); j++) {
                 PyObject *item = PyList_GET_ITEM(a_l_i, j);
@@ -865,9 +1026,15 @@ cPalabra_compute_distances(PyObject *self, PyObject *args) {
                 char *c_c = PyString_AS_STRING(py_c);
                 if (c == *c_c) {
                     count += j;
+                    // TODO use number of times that character occurs as well
+                    occurs = 1;
                     break;
                 }
             }
+            // penalize score if character does not occur at all
+            if (!occurs) {
+                count += PyList_GET_SIZE(a_l_i);
+            }
         }
         PyObject* item = Py_BuildValue("(si)", word, count);
         PyList_Append(result, item);
@@ -925,8 +1092,12 @@ cPalabra_compute_counts(PyObject *self, PyObject *args) {
         PyObject *l_words = PyDict_GetItem(words, key);
         Py_ssize_t w;
         for (w = 0; w < PyList_Size(l_words); w++) {
-            PyObject *item = PyList_GetItem(l_words, w);
-            char *word = PyString_AsString(item);
+            PyObject* item = PyList_GET_ITEM(l_words, w);
+            PyObject* word_str;
+            const int word_score;
+            if (!PyArg_ParseTuple(item, "Oi", &word_str, &word_score))
+                return NULL;
+            char *word = PyString_AsString(word_str);
             int i;
             for (i = 0; i < strlen(word); i++) {
                 char c = *(word + i);
@@ -1062,6 +1233,18 @@ cPalabra_verify_contained_words(PyObject *self, PyObject *args) {
     return result;
 }
 
+static PyObject*
+cPalabra_update_score(PyObject *self, PyObject *args) {
+    PyObject *word;
+    const int word_length;
+    const int score;
+    const int wlist_index;
+    if (!PyArg_ParseTuple(args, "Oiii", &word, &word_length, &score, &wlist_index))
+        return NULL;
+    update_score(trees[wlist_index][word_length], PyString_AsString(word), score);
+    return Py_None;
+}
+
 static PyMethodDef methods[] = {
     {"search", cPalabra_search, METH_VARARGS, "search"},
     {"preprocess", cPalabra_preprocess, METH_VARARGS, "preprocess"},
@@ -1076,6 +1259,8 @@ static PyMethodDef methods[] = {
     {"compute_counts", cPalabra_compute_counts, METH_VARARGS, "compute_counts"},
     {"get_contained_words", cPalabra_get_contained_words, METH_VARARGS, "get_contained_words"},
     {"verify_contained_words", cPalabra_verify_contained_words, METH_VARARGS, "verify_contained_words"},
+    {"update_score", cPalabra_update_score, METH_VARARGS, "update_score"},
+    {"insert_word", cPalabra_insert_word, METH_VARARGS, "insert_word"},
     {NULL, NULL, 0, NULL}
 };
 
