@@ -26,7 +26,9 @@ from itertools import chain
 import action
 from appearance import CellPropertiesDialog
 import constants
+from gui_common import launch_dialog
 from grid import Grid, decompose_word
+import preferences
 from preferences import read_pref_color
 import transform
 from view import GridPreview, DEFAULTS_CELL
@@ -35,6 +37,7 @@ from word import (CWordList,
     analyze_words,
 )
 import cPalabra
+from gui_debug import FillDebugDialog
 
 DEFAULT_FILL_OPTIONS = {
     constants.FILL_OPTION_START: constants.FILL_START_AT_AUTO
@@ -44,9 +47,30 @@ DEFAULT_FILL_OPTIONS = {
 }
 
 Selection = namedtuple('Selection', ['x', 'y', 'direction'])
+EditorAction = namedtuple('EditorAction', ['type', 'args'])
 
 mouse_buttons_down = [False, False, False]
 
+class EditorSettings:
+    def __init__(self):
+        self.surface = None
+        self.pattern = None
+        self.settings = {
+            "symmetries": constants.SYM_180
+            , "locked_grid": False
+        }
+        self.warnings = {}
+        for w in constants.WARNINGS:
+            self.warnings[w] = False
+        self.force_redraw = True
+        self.reset_controls()
+        
+    def reset_controls(self):
+        self.selection = Selection(-1, -1, "across")
+        self.current = (-1, -1)
+e_settings = EditorSettings()
+e_tools = {}
+
 def get_char_slots(grid, c):
     return [(x, y, "across", 1) for x, y in grid.cells() if grid.data[y][x]["char"] == c]
     
@@ -116,6 +140,10 @@ def compute_word_cells(grid, word, x, y, d):
     return [(x, y, c.upper()) for x, y, c in result if grid.data[y][x]["char"] == ""]
 
 def compute_search_args(grid, slot, force=False):
+    """
+    Compute the arguments for searching words,
+    based on the given grid and slot.
+    """
     x, y, d = slot
     if not grid.is_available(x, y):
         return None
@@ -180,10 +208,12 @@ def compute_highlights(grid, f=None, arg=None, clear=False):
         elif f == "slot":
             x, y, d = arg
             cells = [(x, y, d, grid.word_length(x, y, d))]
+        elif f == "slots":
+            cells = [(x, y, d, grid.word_length(x, y, d)) for x, y, d in arg]
     return cells
 
 def compute_warnings_of_cells(grid, cells, settings):
-    """Determine undesired cells."""
+    """Determine undesired cells based on warning settings."""
     lengths = {}
     starts = {}
     warn_unchecked = settings[constants.WARN_UNCHECKED]
@@ -293,30 +323,14 @@ def _render_cells(puzzle, cells, e_settings, drawing_area, editor=True):
     context.set_source(e_settings.pattern)
     context.paint()
 
-class EditorSettings:
-    def __init__(self):
-        self.surface = None
-        self.pattern = None
-        self.selection = Selection(-1, -1, "across")
-        self.current = (-1, -1)
-        self.settings = {
-            "symmetries": constants.SYM_180
-            , "locked_grid": False
-        }
-        self.warnings = {}
-        for w in constants.WARNINGS:
-            self.warnings[w] = False
-e_settings = EditorSettings()
-e_tools = {}
-
-def on_button_release_event(drawing_area, event):
+def on_button_release_event(drawing_area, event, window, puzzle, e_settings):
     if 1 <= event.button <= 3:
         mouse_buttons_down[event.button - 1] = False
     return True
 
 # needed to capture the press of a tab button
 # so focus won't switch to the toolbar
-def on_key_press_event(drawing_area, event):
+def on_key_press_event(drawing_area, event, window, puzzle, e_settings):
     return True
 
 def on_key_release_event(drawing_area, event, window, puzzle, e_settings):
@@ -324,76 +338,119 @@ def on_key_release_event(drawing_area, event, window, puzzle, e_settings):
     if ((event.state & gtk.gdk.SHIFT_MASK) or
         (event.state & gtk.gdk.CONTROL_MASK)):
         return True
-    key = event.keyval
-    grid = puzzle.grid
-    if key == gtk.keysyms.BackSpace and not e_settings.settings["locked_grid"]:
-        on_backspace(window, puzzle, e_settings)
-    elif key == gtk.keysyms.Tab:
-        set_selection(window, puzzle, e_settings, other_dir=True)
-    elif key == gtk.keysyms.Home:
-        cell = grid.get_cell_of_slot(e_settings.selection, "start")
-        set_selection(window, puzzle, e_settings, *cell)
-    elif key == gtk.keysyms.End:
-        cell = grid.get_cell_of_slot(e_settings.selection, "end")
-        set_selection(window, puzzle, e_settings, *cell)
-    elif key == gtk.keysyms.Left:
-        apply_selection_delta(window, puzzle, e_settings, -1, 0)
-    elif key == gtk.keysyms.Up:
-        apply_selection_delta(window, puzzle, e_settings, 0, -1)
-    elif key == gtk.keysyms.Right:
-        apply_selection_delta(window, puzzle, e_settings, 1, 0)
-    elif key == gtk.keysyms.Down:
-        apply_selection_delta(window, puzzle, e_settings, 0, 1)
-    elif key == gtk.keysyms.Delete and not e_settings.settings["locked_grid"]:
-        on_delete(window, puzzle, e_settings)
-    elif not e_settings.settings["locked_grid"]:
-        on_typing(window, puzzle, key, e_settings)
+    arrows_change_dir = preferences.prefs[constants.PREF_ARROWS_CHANGE_DIR]
+    actions = determine_editor_actions(puzzle.grid, e_settings.selection, event.keyval, arrows_change_dir)
+    process_editor_actions(window, puzzle, e_settings, actions)
     return True
 
-def on_typing(window, puzzle, keyval, e_settings):
+def determine_editor_actions(grid, selection, key, arrows_change_dir=False):
+    """
+    Determine all actions that need to take place given the current grid,
+    the current selection and the key that the user has pressed.
+    """
+    actions = []
+    if key == gtk.keysyms.BackSpace:
+        actions = on_backspace(grid, selection)
+    elif key == gtk.keysyms.Tab:
+        x, y, d = selection
+        if grid.is_available(x, y):
+            actions = [EditorAction("swapdir", None)]
+    elif key == gtk.keysyms.Home:
+        x, y, d = selection
+        if grid.is_available(x, y):
+            x, y = grid.get_cell_of_slot(selection, "start")
+            actions = [EditorAction("selection", {'x': x, 'y': y})]
+    elif key == gtk.keysyms.End:
+        x, y, d = selection
+        if grid.is_available(x, y):
+            x, y = grid.get_cell_of_slot(selection, "end")
+            actions = [EditorAction("selection", {'x': x, 'y': y})]
+    elif key == gtk.keysyms.Left:
+        actions = apply_selection_delta(grid, selection, -1, 0)
+    elif key == gtk.keysyms.Up:
+        actions = apply_selection_delta(grid, selection, 0, -1)
+    elif key == gtk.keysyms.Right:
+        actions = apply_selection_delta(grid, selection, 1, 0)
+        if arrows_change_dir and selection[2] == "down":
+            actions.append(EditorAction("swapdir", None))
+    elif key == gtk.keysyms.Down:
+        actions = apply_selection_delta(grid, selection, 0, 1)
+        if arrows_change_dir and selection[2] == "across":
+            actions.append(EditorAction("swapdir", None))
+    elif key == gtk.keysyms.Delete:
+        actions = on_delete(grid, selection)
+    else:
+        actions = on_typing(grid, key, selection)
+    return actions
+    
+def process_editor_actions(window, puzzle, e_settings, actions):
+    """
+    Process all the editor actions and apply them to the grid and editor controls.
+    """
+    is_locked = e_settings.settings["locked_grid"]
+    for a in actions:
+        if a.type in ["blocks", "chars"] and is_locked:
+            continue
+        if a.type == "blocks":
+            x = a.args['x']
+            y = a.args['y']
+            status = a.args['status']
+            blocks = transform_blocks(puzzle.grid, e_settings.settings["symmetries"], x, y, status)
+            if not blocks:
+                continue
+            window.transform_grid(transform.modify_blocks, blocks=blocks)
+        elif a.type in ["chars"]:
+            window.transform_grid(transform.modify_chars, chars=a.args['cells'])
+        elif a.type == "selection":
+            x = a.args['x']
+            y = a.args['y']
+            set_selection(window, puzzle, e_settings, x, y)
+        elif a.type == "swapdir":
+            set_selection(window, puzzle, e_settings, other_dir=True)
+        elif a.type == "popup":
+            x = a.args['x']
+            y = a.args['y']
+            button = a.args['button']
+            time = a.args['time']
+            _create_popup_menu(window, puzzle, button, time, x, y)
+        elif a.type == "render":
+            cells = a.args['cells']
+            _render_cells(puzzle, cells, e_settings, window.drawing_area)
+
+def on_typing(grid, keyval, selection):
     """Place an alphabetical character in the grid and move the selection."""
     valid = gtk.keysyms.a <= keyval <= gtk.keysyms.z or keyval == gtk.keysyms.period
     if not valid:
-        return
-    grid = puzzle.grid
-    x, y, direction = e_settings.selection
-    if not grid.is_valid(x, y):
-        return
+        return []
+    x, y, direction = selection
+    if not grid.is_available(x, y):
+        return []
+    actions = []
     if keyval == gtk.keysyms.period:
-        r_transform_blocks(window, puzzle, e_settings, x, y, True)
+        actions.append(EditorAction("blocks", {'x': x, 'y': y, 'status': True}))
     else:
         c = chr(keyval).capitalize()
         if c != grid.get_char(x, y):
-            window.transform_grid(transform.modify_char
-                    , x=x
-                    , y=y
-                    , next_char=c)
-            #self._check_blacklist_for_cell(x, y)
+            actions.append(EditorAction("chars", {'cells': [(x, y, c)]}))
     dx = 1 if direction == "across" else 0
     dy = 1 if direction == "down" else 0
-    apply_selection_delta(window, puzzle, e_settings, dx, dy)
+    nx, ny = x + dx, y + dy
+    if grid.is_available(nx, ny):
+        actions.append(EditorAction("selection", {'x': nx, 'y': ny}))
+    return actions
 
-def r_transform_blocks(window, puzzle, e_settings, x, y, status):
-    """Place or remove a block at (x, y) and its symmetrical cells."""
-    blocks = transform_blocks(puzzle.grid, e_settings.settings["symmetries"], x, y, status)
-    if not blocks:
-        return
-    window.transform_grid(transform.modify_blocks, blocks=blocks)
-    cells = [(x, y) for x, y, status in blocks]
-    _render_cells(puzzle, cells, e_settings, window.drawing_area)
-
-def on_delete(window, puzzle, e_settings):
+def on_delete(grid, selection):
     """Remove the character in the selected cell."""
-    x, y = e_settings.selection.x, e_settings.selection.y
-    if puzzle.grid.get_char(x, y) != "":
-        window.transform_grid(transform.modify_char
-            , x=x
-            , y=y
-            , next_char="")
-        #self._check_blacklist_for_cell(x, y)
-        _render_cells(puzzle, [(x, y)], e_settings, window.drawing_area)
+    x, y, d = selection
+    if grid.get_char(x, y) != "":
+        return [EditorAction("chars", {'cells': [(x, y, '')]})]
+    return []
 
 def compute_selection(prev, x=None, y=None, direction=None, other_dir=False):
+    """
+    Compute a new selection based on the previous selection
+    and the desired modifications.
+    """
     if other_dir:
         direction = {"across": "down", "down": "across"}[prev.direction]
     nx = x if x is not None else prev[0]
@@ -447,45 +504,37 @@ def set_overlay(window, puzzle, e_settings, word=None):
     render = [(x, y) for x, y, c in (old + cells)]
     _render_cells(puzzle, render, e_settings, window.drawing_area)
 
-def apply_selection_delta(window, puzzle, e_settings, dx, dy):
+def apply_selection_delta(grid, selection, dx, dy):
     """Move the selection to an available nearby cell."""
-    nx, ny = e_settings.selection.x + dx, e_settings.selection.y + dy
-    if puzzle.grid.is_available(nx, ny):
-        set_selection(window, puzzle, e_settings, nx, ny)
+    x, y, d = selection
+    nx, ny = x + dx, y + dy
+    if grid.is_available(nx, ny):
+        return [EditorAction("selection", {'x': nx, 'y': ny})]
+    return []
 
-def on_backspace(window, puzzle, e_settings):
+def on_backspace(grid, selection):
     """Remove a character in the current or previous cell."""
-    x, y, direction = e_settings.selection
-    grid = puzzle.grid
-    transform_grid = window.transform_grid
-    modify_char = transform.modify_char
-    
-    # remove character in selected cell if it has one
+    x, y, direction = selection
     if grid.data[y][x]["char"] != "":
-        transform_grid(modify_char, x=x, y=y, next_char="")
-        #self._check_blacklist_for_cell(x, y)
-        _render_cells(puzzle, [(x, y)], e_settings, window.drawing_area)
-    else:
-        # remove character in previous cell if needed and move selection
-        if direction == "across":
-            x -= 1
-        elif direction == "down":
-            y -= 1
-        if grid.is_available(x, y):
-            if grid.data[y][x]["char"] != "":
-                transform_grid(modify_char, x=x, y=y, next_char="")
-            #self._check_blacklist_for_cell(x, y)
-            set_selection(window, puzzle, e_settings, x, y)
+        return [EditorAction("chars", {'cells': [(x, y, '')]})]
+    actions = []
+    x -= (1 if direction == "across" else 0)
+    y -= (1 if direction == "down" else 0)
+    if grid.is_available(x, y):
+        if grid.data[y][x]["char"] != "":
+            actions.append(EditorAction("chars", {'cells': [(x, y, '')]}))
+        actions.append(EditorAction("selection", {'x': x, 'y': y}))
+    return actions
 
-def insert(window, grid, slot, word):
+def insert(grid, slot, word):
     """Insert a word in the selected slot."""
     x, y, d = slot
     if not grid.is_available(x, y):
-        return
+        return []
     cells = compute_word_cells(grid, word, x, y, d)
     if not cells:
-        return
-    window.transform_grid(transform.modify_chars, chars=cells)
+        return []
+    return [EditorAction("chars", {'cells': cells})]
 
 def highlight_cells(window, puzzle, f=None, arg=None, clear=False):
     """
@@ -498,283 +547,192 @@ def highlight_cells(window, puzzle, f=None, arg=None, clear=False):
     render = list(set(expand_slots(old + cells)))
     _render_cells(puzzle, render, e_settings, window.drawing_area)
     return cells
+    
+def on_button_press(grid, event, prev, next):
+    prev_x, prev_y = prev
+    x, y = next
+    if not grid.is_valid(x, y):
+        return [EditorAction("selection", {'x': -1, 'y': -1})]
+    actions = []
+    if (event.state & gtk.gdk.SHIFT_MASK):
+        if event.button in [1, 3]:
+            args = {'x': x, 'y': y, 'status': event.button == 1}
+            actions.append(EditorAction("blocks", args))
+    else:
+        if event.button == 1:
+            # type is needed to assure rapid clicking
+            # doesn't trigger it multiple times
+            if (prev_x, prev_y) == (x, y) and event.type == gtk.gdk._2BUTTON_PRESS:
+                actions.append(EditorAction("swapdir", None))
+            if grid.is_available(x, y):
+                actions.append(EditorAction("selection", {'x': x, 'y': y}))
+        elif event.button == 3:
+            if grid.is_valid(x, y):
+                actions.append(EditorAction("popup", {'button': event.button
+                    , 'time': event.time, 'x': x, 'y': y}))
+                # popup menu right-click should not interfere with
+                # normal editing controls
+                mouse_buttons_down[2] = False
+    return actions
+
+def has_chars(grid, x, y, direction):
+    return any([grid.data[q][p]["char"] != '' for p, q in grid.slot(x, y, direction)])
+
+def view_cell_properties(window, puzzle, x, y):
+    props = {"cell": (x, y), "grid": puzzle.grid, "defaults": {}}
+    for k in DEFAULTS_CELL:
+        props[k] = puzzle.view.properties.style(x, y)[k]
+        props["defaults"][k] = puzzle.view.properties.style()[k]
+    f_done = lambda w: w.gather_appearance()
+    response, appearance = launch_dialog(CellPropertiesDialog, window, props, f_done=f_done)
+    if response == gtk.RESPONSE_OK:
+        puzzle.view.properties.update(x, y, appearance.items())
+        _render_cells(puzzle, [(x, y)], e_settings, window.drawing_area)
+
+def on_clear_slot_select(grid, direction, x, y):
+    sx, sy = grid.get_start_word(x, y, direction)
+    msg = ''.join(["Clear all letters in the slot: "
+        , str(grid.data[sy][sx]["number"]), " "
+        , {"across": "across", "down": "down"}[direction]])
+    return msg
+
+def clearable(grid, slot):
+    return grid.is_part_of_word(*slot) and has_chars(grid, *slot)
+
+def _create_popup_menu(window, puzzle, button, time, x, y):
+    def create_item(title, activate, tooltip):
+        item = gtk.MenuItem(title)
+        item.connect("activate", activate)
+        select = lambda item: window.update_status(constants.STATUS_MENU, tooltip)
+        item.connect("select", select)
+        item.connect("deselect", lambda i: window.pop_status(constants.STATUS_MENU))
+        return item
+    menu = gtk.Menu()
+    on_clear_slot_deselect = lambda item: window.pop_status(constants.STATUS_MENU)
+    activate = lambda i: clear_slot_of(window, puzzle.grid, x, y, "across")
+    tooltip = on_clear_slot_select(puzzle.grid, "across", x, y)
+    item = create_item(u"Clear across slot", activate, tooltip)
+    item.set_sensitive(clearable(puzzle.grid, (x, y, "across")))
+    menu.append(item)
+    activate = lambda i: clear_slot_of(window, puzzle.grid, x, y, "down")
+    tooltip = on_clear_slot_select(puzzle.grid, "down", x, y)
+    item = create_item(u"Clear down slot", activate, tooltip)
+    item.set_sensitive(clearable(puzzle.grid, (x, y, "down")))
+    menu.append(item)
+    menu.append(gtk.SeparatorMenuItem())
+    activate = lambda i: view_cell_properties(window, puzzle, x, y)
+    item = create_item(u"Properties", activate, u"View properties of this cell")
+    menu.append(item)
+    menu.show_all()
+    menu.popup(None, None, None, button, time)
+
+def clear_slot_of(window, grid, x, y, d):
+    """Clear all letters of the slot in the specified direction
+    that contains (x, y)."""
+    chars = [(r, s, "") for r, s in grid.slot(x, y, d) if grid.data[s][r]["char"] != '']
+    if len(chars) > 0:
+        window.transform_grid(transform.modify_chars, chars=chars)
+
+def on_button_press_event(drawing_area, event, window, puzzle, e_settings):
+    if 1 <= event.button <= 3:
+        mouse_buttons_down[event.button - 1] = True
+    drawing_area.grab_focus()
+    prev_x, prev_y = e_settings.selection.x, e_settings.selection.y
+    x, y = puzzle.view.properties.screen_to_grid(event.x, event.y)
+    actions = on_button_press(puzzle.grid, event, (prev_x, prev_y), (x, y))
+    process_editor_actions(window, puzzle, e_settings, actions)
+    return True
+
+def on_motion_notify_event(drawing_area, event, window, puzzle, e_settings):
+    if event.is_hint:
+        ex, ey, estate = event.window.get_pointer()
+    else:
+        ex, ey, estate = event.x, event.y, event.state
+    cx, cy = puzzle.view.properties.screen_to_grid(ex, ey)
+    previous = e_settings.current
+    e_settings.current = (cx, cy)
+    symms = e_settings.settings["symmetries"]
+    actions = compute_motion_actions(puzzle, symms, previous, (cx, cy)
+        , estate & gtk.gdk.SHIFT_MASK, mouse_buttons_down)
+    process_editor_actions(window, puzzle, e_settings, actions)
+    return True
+    
+def compute_motion_actions(puzzle, symmetries, previous, current, shift_down, mouse_buttons_down):
+    """Compute all editor actions that take place when mouse cursor is moved."""
+    actions = []
+    if previous != current:
+        c0 = apply_symmetry(puzzle.grid, symmetries, *previous)
+        c1 = apply_symmetry(puzzle.grid, symmetries, *current)
+        cells = c0 + c1 + [previous, current]
+        actions.append(EditorAction("render", {'cells': cells}))
+    if shift_down:
+        cx, cy = current
+        if mouse_buttons_down[0] and not mouse_buttons_down[2]:
+            actions.append(EditorAction("blocks", {'x': cx, 'y': cy, 'status': True}))
+        elif mouse_buttons_down[2] and not mouse_buttons_down[0]:
+            actions.append(EditorAction("blocks", {'x': cx, 'y': cy, 'status': False}))
+    return actions
+
+def on_expose_event(drawing_area, event, window, puzzle, e_settings):
+    """Render the main editing component."""
+    if not e_settings.surface or e_settings.force_redraw:
+        grid, view = puzzle.grid, puzzle.view
+        width, height = view.properties.visual_size(True)
+        e_settings.surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
+        e_settings.pattern = cairo.SurfacePattern(e_settings.surface)
+        # TODO should not be needed
+        view.grid = grid
+        e_settings.force_redraw = False
+        _render_cells(puzzle, list(grid.cells()), e_settings, drawing_area, True)
+    context = drawing_area.window.cairo_create()
+    context.set_source(e_settings.pattern)
+    context.paint()
+    return True
+
+EDITOR_EVENTS = {
+    "expose_event": on_expose_event
+    , "button_press_event": on_button_press_event
+    , "button_release_event": on_button_release_event
+    , "motion_notify_event": on_motion_notify_event
+    , "key_press_event": on_key_press_event
+    , "key_release_event": on_key_release_event
+}
+
+def compute_words(grid, wordlists, selection, force_refresh=False, options=None):
+    """Compute the words that should be displayed in the main words control."""
+    args = compute_search_args(grid, selection, force_refresh)
+    if args:
+        return search_wordlists(wordlists, *args, options=options)
+    return []
 
 class Editor:
     def __init__(self, window):
         self.window = window
         self.blacklist = []
         self.fill_options = {}
-        self.EVENTS = {"expose_event": self.on_expose_event
-            , "button_press_event": self.on_button_press_event
-            , "button_release_event": on_button_release_event
-            , "motion_notify_event": self.on_motion_notify_event
-            , "key_press_event": on_key_press_event
-            , "key_release_event": (on_key_release_event, self.window, self.puzzle, e_settings)
-        }
-        self.force_redraw = True
-        
-    def get_puzzle(self):
-        return self.window.puzzle_manager.current_puzzle
-        
-    puzzle = property(get_puzzle)
-    
-    def _render_cells(self, cells, editor=True):
-        _render_cells(self.puzzle, cells, e_settings, self.window.drawing_area, editor)
-        
-    def on_expose_event(self, drawing_area, event):
-        """Render the main editing component."""
-        if not e_settings.surface or self.force_redraw:
-            width, height = self.puzzle.view.properties.visual_size(True)
-            e_settings.surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
-            e_settings.pattern = cairo.SurfacePattern(e_settings.surface)
-            # TODO should not be needed
-            self.puzzle.view.grid = self.puzzle.grid
-            self.force_redraw = False
-            self._render_cells(list(self.puzzle.grid.cells()), editor=True)
-        context = self.window.drawing_area.window.cairo_create()
-        context.set_source(e_settings.pattern)
-        context.paint()
-        return True
-        
-    def on_button_press_event(self, drawing_area, event):
-        if 1 <= event.button <= 3:
-            mouse_buttons_down[event.button - 1] = True
-        drawing_area.grab_focus()
-        prev_x, prev_y = e_settings.selection.x, e_settings.selection.y
-        x, y = self.puzzle.view.properties.screen_to_grid(event.x, event.y)
-        
-        if not self.puzzle.grid.is_valid(x, y):
-            self.set_selection(-1, -1)
-            return True
-            
-        if (event.state & gtk.gdk.SHIFT_MASK):
-            if event.button in [1, 3] and not e_settings.settings["locked_grid"]:
-                self.transform_blocks(x, y, event.button == 1)
-        else:
-            if event.button == 1:
-                # type is needed to assure rapid clicking
-                # doesn't trigger it multiple times
-                if (prev_x, prev_y) == (x, y) and event.type == gtk.gdk._2BUTTON_PRESS:
-                    self.set_selection(other_dir=True)
-                if self.puzzle.grid.is_available(x, y):
-                    self.set_selection(x, y)
-            elif event.button == 3:
-                if self.puzzle.grid.is_valid(x, y):
-                    self._create_popup_menu(event, x, y)
-                    # popup menu right-click should not interfere with
-                    # normal editing controls
-                    mouse_buttons_down[2] = False
-        return True
-    
-    def _create_popup_menu(self, event, x, y):
-        menu = gtk.Menu()
-        update_status = self.window.update_status
-        pop_status = self.window.pop_status
-        def on_clear_slot_select(item, direction, x, y):
-            grid = self.puzzle.grid
-            sx, sy = grid.get_start_word(x, y, direction)
-            msg = ''.join(["Clear all letters in the slot: "
-                , str(grid.data[sy][sx]["number"]), " "
-                , {"across": "across", "down": "down"}[direction]])
-            update_status(constants.STATUS_MENU, msg)
-        on_clear_slot_deselect = lambda item: pop_status(constants.STATUS_MENU)
-        on_clear_slot = lambda item, d: self.clear_slot_of(x, y, d)
-        def has_chars(x, y, direction):
-            grid = self.puzzle.grid
-            return any([grid.data[q][p]["char"] != ''
-                for p, q in grid.slot(x, y, direction)])
-        clearable = lambda slot: self.puzzle.grid.is_part_of_word(*slot) and has_chars(*slot)
-        item = gtk.MenuItem("Clear across slot")
-        item.connect("activate", on_clear_slot, "across")
-        item.connect("select", on_clear_slot_select, "across", x, y)
-        item.connect("deselect", on_clear_slot_deselect)
-        item.set_sensitive(clearable((x, y, "across")))
-        menu.append(item)
-        item = gtk.MenuItem("Clear down slot")
-        item.connect("activate", on_clear_slot, "down")
-        item.connect("select", on_clear_slot_select, "down", x, y)
-        item.connect("deselect", on_clear_slot_deselect)
-        item.set_sensitive(clearable((x, y, "down")))
-        menu.append(item)
-        menu.append(gtk.SeparatorMenuItem())
-        def on_cell_properties(item):
-            puzzle = self.puzzle
-            grid = puzzle.grid
-            def determine_type(c):
-                if grid.is_block(*c):
-                    return "block"
-                elif grid.is_void(*c):
-                    return "void"
-                return "letter"
-            props = {"cell": (x, y), "grid": grid, "defaults": {}}
-            for k in DEFAULTS_CELL:
-                props[k] = puzzle.view.properties.style(x, y)[k]
-                props["defaults"][k] = puzzle.view.properties.style()[k]
-            w = CellPropertiesDialog(self.window, props)
-            w.show_all()
-            if w.run() == gtk.RESPONSE_OK:
-                puzzle.view.properties.update(x, y, w.gather_appearance().items())
-                self._render_cells([(x, y)])
-            w.destroy()
-        item = gtk.MenuItem("Properties")
-        item.connect("activate", on_cell_properties)
-        menu.append(item)
-        menu.show_all()
-        menu.popup(None, None, None, event.button, event.time)
-        
-    def on_motion_notify_event(self, drawing_area, event):
-        if event.is_hint:
-            ex, ey, estate = event.window.get_pointer()
-        else:
-            ex, ey, estate = event.x, event.y, event.state
-        props = self.puzzle.view.properties
-        cx, cy = props.screen_to_grid(ex, ey)
-        prev_x, prev_y = e_settings.current
-        e_settings.current = (cx, cy)
-
-        if (prev_x, prev_y) != (cx, cy):
-            grid = self.puzzle.grid
-            symms = e_settings.settings["symmetries"]
-            c0 = apply_symmetry(grid, symms, prev_x, prev_y)
-            c1 = apply_symmetry(grid, symms, cx, cy)
-            self._render_cells(c0 + c1 + [(prev_x, prev_y), (cx, cy)])
-        
-        transform_blocks = self.transform_blocks
-        if (estate & gtk.gdk.SHIFT_MASK and not e_settings.settings["locked_grid"]):
-            if mouse_buttons_down[0]:
-                transform_blocks(cx, cy, True)
-            elif mouse_buttons_down[2]:
-                transform_blocks(cx, cy, False)
-        return True
-        
-    def clear_slot_of(self, x, y, direction):
-        """Clear all letters of the slot in the specified direction
-        that contains (x, y)."""
-        grid = self.puzzle.grid
-        chars = [(r, s, "") for r, s in grid.slot(x, y, direction)
-            if grid.data[s][r]["char"] != '']
-        if len(chars) > 0:
-            self.window.transform_grid(transform.modify_chars, chars=chars)
-        
-    def refresh_clues(self):
-        """Reload all the word/clue items and select the currently selected item."""
-        p, q = self.puzzle.grid.get_start_word(*e_settings.selection)
-        e_tools["clue"].load_items(self.puzzle)
-        e_tools["clue"].select(p, q, e_settings.selection[2])
-        
-    def refresh_words(self, force_refresh=False):
-        """
-        Update the list of words according to active constraints of letters
-        and the current settings (e.g., show only words with intersections).
-        """
-        args = compute_search_args(self.puzzle.grid, e_settings.selection, force_refresh)
-        if args:
-            result = search_wordlists(self.window.wordlists, *args)
-        else:
-            result = []
-        e_tools["word"].display_words(result)
+        self.fill_options.update(DEFAULT_FILL_OPTIONS)
         
     def fill(self):
         for wlist in self.window.wordlists:
-            results = fill(self.puzzle.grid, wlist.words, self.fill_options)
+            #backup = copy.deepcopy(self.window.puzzle.grid)
+            results = fill(self.window.puzzle.grid, wlist.words, self.fill_options)
+            if False:
+                w = FillDebugDialog(self.window, [backup] + results)
+                w.show_all()
+                w.run()
+                w.destroy()
+                break
             self.window.transform_grid(transform.modify_chars, chars=results[0])
             break
-            
-    def clue(self, x, y, direction, key, value):
-        """
-        Update the clue data by creating or updating the latest undo action.
-        """
-        self.window.transform_clues(transform.modify_clue
-                , x=x
-                , y=y
-                , direction=direction
-                , key=key
-                , value=value)
         
     def insert(self, word):
         """Insert a word in the selected slot."""
-        if e_settings.settings["locked_grid"]:
-            return
-        insert(self.window, self.puzzle.grid, e_settings.selection, word)
+        actions = insert(self.window.puzzle.grid, e_settings.selection, word)
+        process_editor_actions(self.window, self.window.puzzle, e_settings, actions)
             
     def set_overlay(self, word=None):
         """
         Display the word in the selected slot without storing it the grid.
         If the word is None, the overlay will be cleared.
         """
-        set_overlay(self.window, self.puzzle, e_settings, word)
-            
-    def transform_blocks(self, x, y, status):
-        """Place or remove a block at (x, y) and its symmetrical cells."""
-        r_transform_blocks(self.window, self.puzzle, e_settings, x, y, status)
-            
-    def _check_blacklist_for_cell(self, x, y):
-        """
-        Check whether the cell (x, y) is part of a blacklisted word.
-        The blacklist is updated accordingly.
-        """
-        return # TODO until ready
-        def get_segment(direction, x, y, dx, dy):
-            """Gather the content of the cells touching and including (x, y)."""
-            segment = []
-            for p, q in self.puzzle.grid.in_direction(x + dx, y + dy, direction):
-                c = self.puzzle.grid.get_char(p, q)
-                if not c:
-                    break
-                segment.append((p, q, c))
-            segment.insert(0, (x, y, self.puzzle.grid.get_char(x, y)))
-            for p, q in self.puzzle.grid.in_direction(x - dx, y - dy, direction, reverse=True):
-                c = self.puzzle.grid.get_char(p, q)
-                if not c:
-                    break
-                segment.insert(0, (p, q, c))
-            return direction, segment
-        def check_segment(direction, segment):
-            """Determine the cells that need to be blacklisted."""
-            result = []
-            word = "".join([c.lower() if c else " " for x, y, c in segment])
-            badwords = self.window.blacklist.get_substring_matches(word)
-            for i in xrange(len(word)):
-                for b in badwords:
-                    if word[i:i + len(b)] == b:
-                        p, q, c = segment[i]
-                        result.append((p, q, direction, len(b)))
-            return result
-        def clear_blacklist(direction, segment):
-            """Remove all blacklist entries related to cells in data."""
-            remove = []
-            for p, q, bdir, length in self.blacklist:
-                for r, s, c in segment:
-                    if (p, q, bdir) == (r, s, direction):
-                        remove.append((p, q, bdir, length))
-            for x in remove:
-                self.blacklist.remove(x)
-        across = get_segment("across", x, y, 1, 0)
-        down = get_segment("down", x, y, 0, 1)
-        for data in [across, down]:
-            clear_blacklist(*data)
-            self.blacklist.extend(check_segment(*data))
-        
-    def refresh_visual_size(self):
-        # TODO fix design
-        self.puzzle.view.grid = self.puzzle.grid
-        self.puzzle.view.properties.grid = self.puzzle.grid
-        size = self.puzzle.view.properties.visual_size()
-        self.window.drawing_area.set_size_request(*size)
-
-    def set_selection(self
-        , x=None
-        , y=None
-        , direction=None
-        , full_update=True
-        , other_dir=False
-        , selection_changed=True):
-        """
-        Select (x, y), the direction or both.
-        Use other_dir to switch the typing direction to the other direction.
-        """
-        set_selection(self.window, self.puzzle, e_settings, x, y, direction, full_update, other_dir, selection_changed)
-        
-    def get_selection(self):
-        """Return the (x, y) of the selected cell."""
-        return (e_settings.selection.x, e_settings.selection.y)
+        set_overlay(self.window, self.window.puzzle, e_settings, word)
