@@ -17,7 +17,15 @@
 
 import gtk
 
-SETTING_TABS = [("page", u"Page"), ("grid", u"Grid"), ("clue", u"Clue")]
+import constants
+import files
+
+SETTING_TABS = [("page", u"Page")
+    , ("grid", u"Grid")
+    , ("clue", u"Clue")
+    , ("clueh", u"Clue header")
+    , ("answers", u"Answers")
+]
 OUTPUT_OPTIONS = [("puzzle", u"Puzzle")
     , ("grid", u"Grid")
     , ("solution", u"Solution")
@@ -48,32 +56,83 @@ class Setting:
         self.editable = editable
 
 class HeaderEditor(gtk.Dialog):
-    def __init__(self, palabra_window):
+    def __init__(self, parent, puzzle, header):
         flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT
-        super(HeaderEditor, self).__init__(u"Page header editor", palabra_window, flags)
-        
-        hbox = gtk.HBox(False, 0)
-        hbox.set_border_width(12)
-        hbox.set_spacing(18)
-        main = gtk.HBox(False, 0)
+        super(HeaderEditor, self).__init__(u"Page header editor", parent, flags)
+        self.puzzle = puzzle
+        self.header = header
+        main = gtk.VBox()
+        main.set_border_width(12)
         main.set_spacing(18)
-        hbox.pack_start(main, True, True, 0)
+        label = gtk.Label()
+        label.set_markup(u"<b>Header text</b>:")
+        label.set_alignment(0, 0.5)
+        main.pack_start(label, False, False, 0)
+        self.text = gtk.TextView()
+        self.text.set_size_request(512, 320)
+        self.text.get_buffer().connect("changed", self.on_header_changed)
+        main.pack_start(self.text, True, True, 0)
+        label = gtk.Label()
+        label.set_markup(u"<b>Header preview</b>:")
+        label.set_alignment(0, 0.5)
+        main.pack_start(label, False, False, 0)
+        self.preview = gtk.Label()
+        s_window = gtk.ScrolledWindow()
+        s_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        s_window.add_with_viewport(self.preview)
+        main.pack_start(s_window, False, False, 0)
+        self.text.get_buffer().set_text(self.header)
+        labels_vbox = gtk.VBox()
+        labels_vbox.set_border_width(12)
+        labels_vbox.set_spacing(6)
+        label = gtk.Label()
+        label.set_markup(u"<b>Click to insert</b>:")
+        label.set_alignment(0, 0.5)
+        labels_vbox.pack_start(label, False, False, 0)
+        for code, title in constants.META_CAPTIONS:
+            label = gtk.Label(title + " (" + code + ")")
+            label.set_alignment(0, 0.5)
+            eventbox = gtk.EventBox()
+            eventbox.add(label)
+            eventbox.connect("button-press-event", self.on_code_clicked, code)
+            labels_vbox.pack_start(eventbox, False, False, 0)
+        content_box = gtk.HBox()
+        content_box.pack_start(main, True, True, 0)
+        content_box.pack_start(labels_vbox, False, False, 0)
+        self.vbox.pack_start(content_box, True, True, 0)
+        self.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
+        self.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
         
-        text = gtk.TextView()
-        text.set_size_request(320, 200)
-        main.pack_start(text, False, False, 0)
+    def on_code_clicked(self, widget, event, code):
+        buff = self.text.get_buffer()
+        start, end = buff.get_bounds()
+        header = buff.get_text(start, end)
+        insert_it = buff.get_iter_at_mark(buff.get_insert())
+        if insert_it.get_char() == buff.get_end_iter().get_char():
+            if header.endswith(" "):
+                buff.set_text(header + code)
+            else:
+                buff.set_text(header + " " + code)
+        else:
+            buff.insert_at_cursor(code)
         
-        self.vbox.pack_start(hbox, True, True, 0)
+    def on_header_changed(self, buff):
+        start, end = buff.get_bounds()
+        self.header = buff.get_text(start, end).strip()
+        preview_txt = files.compute_header(self.puzzle, self.header)
+        preview_txt = files.compute_header(self.puzzle, preview_txt, page_n=0)
+        self.preview.set_text(preview_txt)
 
 class ExportWindow(gtk.Dialog):
-    def __init__(self, palabra_window):
+    def __init__(self, palabra_window, puzzle):
         flags = gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT
         super(ExportWindow, self).__init__(u"Export puzzle", palabra_window, flags)
         self.palabra_window = palabra_window
+        self.puzzle = puzzle
         pdf = Format("pdf", u"PDF (pdf)", ["puzzle", "grid", "solution", "answers"])
         pdf.add(Setting("page", "bool", u"Include header", "page_header_include", True))
         pdf.add(Setting("page", "bool", u"Include header on each page", "page_header_include_all", False))
-        pdf.add(Setting("page", "text", u"Header:", "page_header_text", u"%T / %A"))#, editable=self.on_edit_header))
+        pdf.add(Setting("page", "text", u"Header:", "page_header_text", u"%T / %A", editable=self.on_edit_header))
         pdf.add(Setting("grid", "spin", u"Cell size in puzzle (mm)", "cell_size_puzzle", 7, (5, 10)))
         pdf.add(Setting("grid", "spin", u"Cell size in solution (mm)", "cell_size_solution", 6, (5, 10)))
         pdf.add(Setting("grid", "combo", u"Align grid:", "align", "right"
@@ -81,6 +140,7 @@ class ExportWindow(gtk.Dialog):
         pdf.add(Setting("clue", "combo", u"Clues:", "clue_placement", "wrap"
             , [(u"Wrapped around grid", "wrap"), (u"Below grid", "below")]))
         pdf.add(Setting("clue", "spin", u"Columns", "n_columns", 3, (3, 5)))
+        pdf.add(Setting("clue", "spin", u"Spacing between columns\n\t(% of available width)", "column_spacing", 5, (0, 20)))
         pdf.add(Setting("page", "spin", u"Margin left (mm)", "margin_left", 20, (0, 50)))
         pdf.add(Setting("page", "spin", u"Margin right (mm)", "margin_right", 20, (0, 50)))
         pdf.add(Setting("page", "spin", u"Margin top (mm)", "margin_top", 20, (0, 50)))
@@ -88,15 +148,17 @@ class ExportWindow(gtk.Dialog):
         pdf.add(Setting("clue", "bool", u"Bold clue number", "clue_number_bold", True))
         pdf.add(Setting("clue", "bool", u"Add period after clue number", "clue_number_period", False))
         pdf.add(Setting("clue", "bool", u"Add length of solution after clue", "clue_length", False))
-        pdf.add(Setting("clue", "text", u"Across header:", "clue_header_across", u"Across"))
-        pdf.add(Setting("clue", "text", u"Down header:", "clue_header_down", u"Down"))
-        pdf.add(Setting("clue", "bool", u"Bold clue header", "clue_header_bold", True))
-        pdf.add(Setting("clue", "bool", u"Italic clue header", "clue_header_italic", False))
-        pdf.add(Setting("clue", "bool", u"Underline clue header", "clue_header_underline", False))
+        pdf.add(Setting("clueh", "text", u"Across header:", "clue_header_across", u"Across"))
+        pdf.add(Setting("clueh", "text", u"Down header:", "clue_header_down", u"Down"))
+        pdf.add(Setting("clueh", "bool", u"Bold clue header", "clue_header_bold", True))
+        pdf.add(Setting("clueh", "bool", u"Italic clue header", "clue_header_italic", False))
+        pdf.add(Setting("clueh", "bool", u"Underline clue header", "clue_header_underline", False))
+        pdf.add(Setting("answers", "spin", u"Width of clue column\n\t(% of available width)", "answers_clue_width", 50, (10, 90)))
+        pdf.add(Setting("answers", "spin", u"Width of spacing between clues / answers\n\t(% of available width)", "answers_clue_sep", 5, (0, 20)))
         png = Format("png", u"PNG (png)", ["grid", "solution"], False)
         self.formats = [pdf, png]
         self.outputs = {}
-        self.format = None
+        self.format = None        
         
         items = gtk.ListStore(str)
         for format in self.formats:
@@ -187,7 +249,7 @@ class ExportWindow(gtk.Dialog):
             tab.set_border_width(6)
             tab.set_spacing(6)
             tab_c = gtk.VBox(False, 0)
-            self._create_settings(tab_c, key_setts)
+            self._create_settings(tab_c, key_setts, self.puzzle)
             tab.pack_start(tab_c, True, True, 0)
             self.tabs.append_page(tab, gtk.Label(t))
         self.option = gtk.VBox(False, 0)
@@ -233,10 +295,12 @@ class ExportWindow(gtk.Dialog):
         except AttributeError:
             pass
             
-    def on_edit_header(self, item):
-        w = HeaderEditor(self.palabra_window)
+    def on_edit_header(self, item, puzzle):
+        cur_header = self.options["settings"]["page_header_text"]
+        w = HeaderEditor(self, puzzle, cur_header)
         w.show_all()
-        w.run()
+        if w.run() == gtk.RESPONSE_OK:
+            self.options["settings"]["page_header_text"] = w.header
         w.destroy()
             
     def _create_output_options(self, main, format, callback):
@@ -269,12 +333,12 @@ class ExportWindow(gtk.Dialog):
         self.check_ok_button(check=format.allow_multiple)
         
     @staticmethod
-    def _create_settings(main, settings):
+    def _create_settings(main, settings, puzzle):
         if not settings:
             return
-        table = gtk.Table(len(settings), 3)
+        table = gtk.Table(len(settings), 2)
         table.set_col_spacings(6)
-        table.set_row_spacings(3)
+        table.set_row_spacings(6)
         main.pack_start(table, False, False, 0)
         for row, s in enumerate(settings):
             if s.type == "combo":
@@ -304,12 +368,13 @@ class ExportWindow(gtk.Dialog):
                 align.set_padding(0, 0, 12, 0)
                 align.add(gtk.Label(s.title))
                 table.attach(align, 0, 1, row, row + 1, gtk.FILL, gtk.FILL)
-                table.attach(widget, 1, 2, row, row + 1)
                 if s.editable:
                     button = gtk.ToolButton() # TODO check
                     button.set_stock_id(gtk.STOCK_EDIT)
-                    button.connect("clicked", s.editable)
-                    table.attach(button, 2, 3, row, row + 1)
+                    button.connect("clicked", s.editable, puzzle)
+                    table.attach(button, 1, 2, row, row + 1)
+                else:
+                    table.attach(widget, 1, 2, row, row + 1)
             elif s.type in ["combo", "spin"]:
                 align = gtk.Alignment(0, 0.5)
                 align.set_padding(0, 0, 12, 0)
