@@ -116,7 +116,7 @@ def determine_file_type(filename):
         try:
             doc = etree.parse(filename)
         except etree.XMLSyntaxError:
-            raise ParserError(u"This is not an XML file.")
+            raise ParserError(u"This is not a valid Palabra puzzle file.")
         root = doc.getroot()
         if root.tag == "Puzzles":
             t = constants.PUZZLE_XPF
@@ -235,7 +235,7 @@ class PangoCairoTable():
                 return True, offset + i
             r_y = self.render(context, 0, y, r_1, wrap=True)
             if r_2 is not None:
-                self.render(context, 1, y, r_2)
+                self.render(context, 2, y, r_2)
             y += r_y #(r_y / 2) # TODO why divide by 2
         return True, None
         
@@ -275,6 +275,8 @@ def export_to_pdf(puzzle, filename, outputs, settings):
         layout.set_markup(''.join(text))
         context.move_to(margin_left, margin_top)
         pcr.show_layout(layout)
+        w, h = layout.get_pixel_size()
+        return h
     def produce_clues(clue_break=False, answers=False, reduce_to_rows=False):
         content = {"clue_markup": {}, "clues": {}}
         c_h = settings["clue_header"]
@@ -422,7 +424,7 @@ def export_to_pdf(puzzle, filename, outputs, settings):
         for p, x, y, w, h, text in r_columns:
             if p not in pages:
                 pages.append(p)
-        def render_page(page):
+        def render_page(page, header_delta=None):
             for p, x, y, w, h, text in r_columns:
                 if p != page:
                     continue
@@ -434,8 +436,13 @@ def export_to_pdf(puzzle, filename, outputs, settings):
                 layout.set_markup(text)
                 pcr.show_layout(layout)
             return True, None
-        return render_page, pages
-    def gen_columns(col_width, padding=None, grid_w=None, grid_h=None, position=None):
+        return [(render_page, p) for p in pages]
+    def gen_columns(col_width
+        , padding=None
+        , grid_w=None
+        , grid_h=None
+        , position=None
+        , header_delta=None):
         clue_placement = settings["clue_placement"]
         n_columns = settings["n_columns"]
         page = 0
@@ -443,7 +450,7 @@ def export_to_pdf(puzzle, filename, outputs, settings):
         pos_x, pos_y = position
         while True:
             for n in xrange(n_columns):
-                col_height = c_height
+                col_height = c_height - header_delta
                 col_x = x + n * col_width + n * padding
                 col_y = y
                 shrink = False
@@ -460,15 +467,15 @@ def export_to_pdf(puzzle, filename, outputs, settings):
                     col_y += (grid_h + padding)
                 yield page, col_x, col_y, col_width, col_height
             page += 1
-    def produce_puzzle(mode, align, cell_size=7, add_clues=False):
-        padding = 20
+    def adjust_grid_props(align, cell_size=7):
+        padding = int(c_width * settings["column_spacing"] / 100)
         n_columns = settings["n_columns"]
+        col_width = int((c_width - ((n_columns - 1) * padding)) / n_columns)
         props = puzzle.view.properties
         prevs = {
             ("cell", "size"): props["cell", "size"]
             , "margin": props.margin
         }
-        col_width = int((c_width - ((n_columns - 1) * padding)) / n_columns)
         props["cell", "size"] = cell_size * mm_unit
         grid_w, grid_h = props.visual_size(False)
         if align == "right":
@@ -477,59 +484,69 @@ def export_to_pdf(puzzle, filename, outputs, settings):
             position = margin_left + (c_width - grid_w) / 2, margin_top
         elif align == "left":
             position = margin_left, margin_top
-        def render_puzzle():
+        return col_width, prevs, grid_w, grid_h, position, padding
+    def produce_puzzle(mode, prevs, position, add_clues=False):
+        def render_puzzle(header_delta=None):
             puzzle.view.properties.margin = position
             puzzle.view.render(context, mode)
             puzzle.view.pdf_reset(prevs)
             return (not add_clues), None
-        result = [(render_puzzle, None)]
-        if add_clues:
-            content = produce_clues(clue_break=True)
-            columns = gen_columns(col_width, padding, grid_w, grid_h, position)
-            f, args = show_clues_columns(content, columns)
-            result += [(f, a) for a in args]
-        return result
+        return [(render_puzzle, None)]
     p_h_include = settings["page_header_include"]
     p_h_all = settings["page_header_include_all"]
     page_n = 0
     header = compute_header(puzzle, settings["page_header_text"])
-    for o in outputs:
+    GRID_ALIGNMENT = {"puzzle": settings["align"]
+        , "grid": "center"
+        , "solution": "center"
+    }
+    CELL_SIZE = {"puzzle": settings["cell_size_puzzle"]
+        , "grid": settings["cell_size_puzzle"]
+        , "solution": settings["cell_size_solution"]
+    }
+    RENDER_MODE = {
+        "puzzle": constants.VIEW_MODE_EXPORT_PDF_PUZZLE
+        , "grid": constants.VIEW_MODE_EXPORT_PDF_PUZZLE
+        , "solution": constants.VIEW_MODE_EXPORT_PDF_SOLUTION
+    }
+    def compute_funcs(header_delta):
+        if o in ["puzzle", "grid", "solution"]:
+            grid_props = adjust_grid_props(GRID_ALIGNMENT[o], CELL_SIZE[o])
+            funcs = produce_puzzle(RENDER_MODE[o], grid_props[1], grid_props[4], o == "puzzle")
         if o == "puzzle":
-            mode = constants.VIEW_MODE_EXPORT_PDF_PUZZLE
-            funcs = produce_puzzle(mode=mode
-                , align=settings["align"]
-                , cell_size=settings["cell_size_puzzle"]
-                , add_clues=True)
-        elif o == "grid":
-            mode = constants.VIEW_MODE_EXPORT_PDF_PUZZLE
-            funcs = produce_puzzle(mode=mode
-                , align="center"
-                , cell_size=settings["cell_size_puzzle"])
-        elif o == "solution":
-            mode = constants.VIEW_MODE_EXPORT_PDF_SOLUTION
-            funcs = produce_puzzle(mode=mode
-                , align="center"
-                , cell_size=settings["cell_size_solution"])
+            columns = gen_columns(grid_props[0]
+                , grid_props[5], grid_props[2], grid_props[3]
+                , grid_props[4], header_delta)
+            funcs += show_clues_columns(produce_clues(clue_break=True), columns)
         elif o == "answers":
             rows = produce_clues(clue_break=True, answers=True, reduce_to_rows=True)
-            columns = [int(0.6 * c_width), int(0.4 * c_width)]
-            table = PangoCairoTable(columns, margin=(margin_left, margin_top))
-            def render_table(offset):
-                return table.render_rows(context, rows, c_height, offset)
+            w1 = int((settings["answers_clue_width"] / 100.0) * c_width)
+            w2 = int((settings["answers_clue_sep"] / 100.0) * c_width)
+            w3 = c_width - w1 - w2
+            table = PangoCairoTable([w1, w2, w3], margin=(margin_left, margin_top))
+            def render_table(offset, header_delta):
+                return table.render_rows(context, rows, c_height - header_delta, offset)
             funcs = [(render_table, 0)]
+        return funcs
+    for o in outputs:
+        funcs = None
         done = False
         count = 0
         while not done:
             context.save()
+            header_delta = 0
             if p_h_include and (True if p_h_all else page_n == 0):
-                pdf_header(compute_header(puzzle, header, page_n))
-                context.translate(0, 24)
+                header_height = pdf_header(compute_header(puzzle, header, page_n))
+                header_delta = header_height + 10
+                context.translate(0, header_delta)
+            if funcs is None:
+                funcs = compute_funcs(header_delta)
             todo = funcs[count:]
-            for p, p_args in todo:
+            for f, f_args in todo:
                 count += 1
-                r, new_arg = p(p_args) if p_args is not None else p()
+                r, new_arg = f(f_args, header_delta) if f_args is not None else f(header_delta)
                 if new_arg is not None:
-                    funcs.append((p, new_arg))
+                    funcs.append((f, new_arg))
                 if not r:
                     continue
                 context.show_page()
@@ -803,17 +820,17 @@ def read_xpf(filename, warnings=True):
                         if warnings:
                             print "Warning: skipping a child of Grid that is not a Row."
                         continue
-                    if not row.text or len(row.text) < r_width:
+                    if not row.text:# or len(row.text) < r_width:
                         if warnings:
                             print "Warning: skipping a row with missing content."
                         continue
                     for x, c in enumerate(row.text):
                         if c == '.':
-                            r_grid.set_block(x, y, True)
+                            r_grid.data[y][x]["block"] = True
                         elif c == '~':
-                            r_grid.set_void(x, y, True)
+                            r_grid.data[y][x]["void"] = True
                         else:
-                            r_grid.set_char(x, y, c if c != ' ' else '')
+                            r_grid.data[y][x]["char"] = c if c != ' ' else ''
                     y += 1
             elif child.tag == "Circles":
                 for circle in child:
