@@ -34,7 +34,7 @@ def determine_scrabble_score(grid):
         , 'H': 4, 'I': 1, 'J': 8, 'K': 5, 'L': 1, 'M': 3, 'N': 1
         , 'O': 1, 'P': 3, 'Q': 10, 'R': 1, 'S': 1, 'T': 1, 'U': 1
         , 'V': 4, 'W': 4, 'X': 8, 'Y': 4, 'Z': 10, '': 0}
-    chars = [grid.get_char(x, y) for x, y in grid.cells()]
+    chars = [grid.data[y][x]["char"] for x, y in grid.cells()]
     return sum([scores[c] for c in chars])
 
 class Grid:
@@ -272,11 +272,20 @@ class Grid:
         percentage = (float(status["block_count"]) / float(self.width * self.height)) * 100
         status["block_percentage"] = percentage
         if full:
-            status["mean_word_length"] = self.mean_word_length()
+            check_counts = self.get_check_count_all()
+            status["mean_word_length"] = self.mean_word_length(check_counts)
             status["blank_count"] = status["char_count"] - status["actual_char_count"]
             status["word_counts"] = self.determine_word_counts()
+            status["checked_count"] = 0
+            status["unchecked_count"] = 0
+            status["clue_count"] = 0
             status["char_counts"] = {}
             for x, y in self.cells():
+                status["clue_count"] += len(self.data[y][x]["clues"])
+                if 0 <= check_counts[x, y] <= 1:
+                    status["unchecked_count"] += 1
+                elif check_counts[x, y] == 2:
+                    status["checked_count"] += 1
                 c = self.data[y][x]["char"]
                 if not c:
                     continue
@@ -285,23 +294,11 @@ class Grid:
                 except KeyError:
                     status["char_counts"][c] = 1
             status["char_counts_total"] = []
+            s_char_counts = status["char_counts"]
             for c in string.ascii_uppercase:
-                try:
-                    count = status["char_counts"][c]
-                except KeyError:
-                    count = 0
+                count = s_char_counts[c] if c in s_char_counts else 0
                 status["char_counts_total"].append((c, count))
-            counts = self.get_check_count_all()
-            status["checked_count"] = 0
-            status["unchecked_count"] = 0
-            status["clue_count"] = 0
-            for x, y in self.cells():
-                if 0 <= counts[x, y] <= 1:
-                    status["unchecked_count"] += 1
-                elif counts[x, y] == 2:
-                    status["checked_count"] += 1
-                status["clue_count"] += len(self.data[y][x]["clues"])
-            status["open_count"] = len(self.compute_open_squares())
+            status["open_count"] = self.compute_open_squares(count=True)
             status["connected"] = self.is_connected()
             status["complete_count"] = self.count_complete()
         return status
@@ -320,7 +317,6 @@ class Grid:
                     status[length] = 1
         count_by_dir("across")
         count_by_dir("down")
-                
         status["total"] = []
         for length in range(2, max(self.width, self.height) + 1):
             try:
@@ -395,12 +391,6 @@ class Grid:
             for x in xrange(self.width):
                 yield x, y
                 
-    def availables(self):
-        """Convenience function for iterating over available cells."""
-        for x, y in self.cells():
-            if self.is_available(x, y):
-                yield x, y
-                
     def gather_word(self, x, y, direction, empty_char=constants.MISSING_CHAR):
         """Return the word starting at (x, y) in the given direction."""
         word = ""
@@ -536,32 +526,34 @@ class Grid:
         width = self.width
         height = self.height
         if diagonals:
-            neighbors = [(dx, dy) for dx in [-1, 0, 1] for dy in [-1, 0, 1]]
-            neighbors.remove((0, 0))
+            neighbors = [(-1, -1), (-1, 0), (-1, 1), (0, -1), (0, 1), (1, -1), (1, 0), (1, 1)]
         else:
             neighbors = [(0, -1), (1, 0), (0, 1), (-1, 0)]
         for dx, dy in neighbors:
             if 0 <= x + dx < width and 0 <= y + dy < height:
                 yield x + dx, y + dy
         
-    def compute_open_squares(self):
+    def compute_open_squares(self, count=False):
         """
-        Return the open squares.
-        
-        A square is open if it does not touch a block, including diagonally.
+        Return a list of open squares in this grid.
+        If count is True, just return the number of open squares.
+        A square is open if it does not touch a block or void, including diagonally.
         """
         data = self.data
         neighbors = self.neighbors
-        cells = []
+        result = 0 if count else []
         for x, y in self.cells():
             if data[y][x]["block"] or data[y][x]["void"]:
                 continue
             for p, q in neighbors(x, y, diagonals=True):
-                if data[q][p]["block"]:
+                if data[q][p]["block"] or data[q][p]["void"]:
                     break
             else:
-                cells.append((x, y))
-        return cells
+                if count:
+                    result += 1
+                else:
+                    result.append((x, y))
+        return result
         
     def is_connected(self):
         """
@@ -569,21 +561,30 @@ class Grid:
         other by going from cell to cell, using only horizontal and
         vertical steps.
         """
-        avs = [(x, y) for x, y in self.availables()]
-        if len(avs) == 0:
+        availables = {}
+        a_cell = None
+        a_count = 0
+        is_available = self.is_available
+        for x, y in self.cells():
+            availables[x, y] = is_available(x, y)
+            if availables[x, y]:
+                if a_cell is None:
+                    a_cell = (x, y)
+                a_count += 1
+        if a_count == 0:
             return True
-    
+        neighbors = self.neighbors
         done = []
-        check = [avs[0]]
+        check = [a_cell]
         while check:
             x, y = check.pop()
             done.append((x, y))
-            for p, q in self.neighbors(x, y):
-                if (p, q) in done or (p, q) in check:
+            for cell in neighbors(x, y):
+                if cell in done or cell in check:
                     continue
-                if self.is_available(p, q):
-                    check.append((p, q))
-        return len(avs) == len(done)
+                if availables[cell]:
+                    check.append(cell)
+        return a_count == len(done)
         
     def count_voids(self):
         """Return the number of voids in the grid."""
@@ -647,12 +648,15 @@ class Grid:
         """Return True iff the grid has at least one character."""
         return self.count_chars(include_blanks=False, _return_at_one=True) == 1
             
-    def mean_word_length(self):
+    def mean_word_length(self, check_counts=None):
         """Return the mean length of the words in the grid."""
         word_count = self.count_words()
         if word_count == 0:
             return 0
-        char_counts = [self.get_check_count(x, y) for x, y in self.availables()]
+        if not check_counts:
+            char_counts = [self.get_check_count(x, y) for x, y in self.cells() if self.is_available(x, y)]
+        else:
+            char_counts = [check_counts[x, y] for x, y in self.cells() if self.is_available(x, y)]
         return float(sum(char_counts)) / float(word_count)
         
     def generate_diagonals(self, direction=0):
@@ -728,7 +732,37 @@ class Grid:
                 slots.append((d, t1))
                 slots.append((opp[d], t2))
         return slots
-            
+        
+    def convert_to_shape(self):
+        """
+        Compute the cells to remove from the grid
+        to create a grid with arbitrary shape.
+        """
+        data = self.data
+        set_void = self.set_void
+        check = [(x, 0) for x in xrange(self.width)]
+        check += [(0, y) for y in xrange(self.height)]
+        check += [(self.width - 1, y) for y in xrange(self.height)]
+        check += [(x, self.height - 1) for x in xrange(self.width)]
+        remove = []
+        done = []
+        neighbors = self.neighbors
+        while check:
+            x, y = check.pop()
+            is_void = data[y][x]["void"]
+            is_block = data[y][x]["block"]
+            if not is_void and not is_block and data[y][x]["char"] == '':
+                remove.append((x, y))
+            done.append((x, y))
+            for cell in neighbors(x, y):
+                if cell in done or cell in check:
+                    continue
+                cx, cy = cell
+                if data[cy][cx]["block"] or data[cy][cx]["char"] != '':
+                    continue
+                check.append(cell)
+        return list(set(remove))
+
     def resize(self, width, height, make_dirty=True):
         """
         Resize the grid to the given dimensions.
@@ -757,10 +791,13 @@ class Grid:
         self.lines = None
         
     def count_complete(self):
+        """Compute the number of complete words for each direction."""
+        in_direction = self.in_direction
+        data = self.data
         r = {"across": 0, "down": 0}
         for n, x, y, d in self.words(allow_duplicates=True, include_dir=True):
-            for p, q in self.in_direction(x, y, d):
-                if self.data[q][p]["char"] == '':
+            for p, q in in_direction(x, y, d):
+                if data[q][p]["char"] == '':
                     break
             else:
                 r[d] += 1
