@@ -112,6 +112,7 @@ _OTHER_COLOR_PREFS = [
 _INT_PREFS = [
     (constants.PREF_INITIAL_HEIGHT, 15)
     , (constants.PREF_INITIAL_WIDTH, 15)
+    , (constants.PREF_FIND_WORD_MIN_SCORE, 0)
 #    , (constants.PREF_UNDO_STACK_SIZE, 50)    
 ]
 for k, color in (_COLOR_ATTRS + _OTHER_COLOR_PREFS):
@@ -122,11 +123,19 @@ for k, color in (_COLOR_ATTRS + _OTHER_COLOR_PREFS):
     _INT_PREFS.extend([(k + "_red", r), (k + "_green", g), (k + "_blue", b)])
 _BOOL_PREFS = [
     (constants.PREF_COPY_BEFORE_SAVE, False)
+    , (constants.PREF_ARROWS_CHANGE_DIR, False)
 #    , (constants.PREF_UNDO_FINITE_STACK, True)
 ]
 _FILE_PREFS = [
     (constants.PREF_WORD_FILES, [PreferenceFile("/usr/share/dict/words", "Default")])
     , (constants.PREF_PATTERN_FILES, [])
+    , (constants.PREF_CLUE_FILES, [])
+]
+_STR_PREFS = [
+    (constants.PREF_BLACKLIST, "")
+]
+_LIST_STR_PREFS = [
+    (constants.PREF_FIND_WORD_FILES, ["/usr/share/dict/words"])
 ]
 
 DEFAULTS = {}
@@ -141,6 +150,10 @@ for code, files in _FILE_PREFS:
             , "name": {"type": "str", "value": f.name}
         })
     DEFAULTS[code] = Preference(result, list, "list", "file")
+for code, s in _STR_PREFS:
+    DEFAULTS[code] = Preference(s, str, "str", None)
+for code, l in _LIST_STR_PREFS:
+    DEFAULTS[code] = Preference(l, list, "list", "str")
 
 def read_config_file(filename=constants.CONFIG_FILE_LOCATION, warnings=True):
     """
@@ -155,6 +168,8 @@ def read_config_file(filename=constants.CONFIG_FILE_LOCATION, warnings=True):
                 for c2 in c:
                     d = {"type": c2.get("type"), "value": c2.text}
                     value[c2.get("name")] = d
+            elif c.get("type") == "str":
+                value = c.text
             values.append(value)
         return values
     props = {}
@@ -167,6 +182,8 @@ def read_config_file(filename=constants.CONFIG_FILE_LOCATION, warnings=True):
             name = p.get("name")
             if t in ["int", "bool"]:
                 props[name] = p.text
+            elif t == "str":
+                props[name] = p.text if p.text is not None else ''
             elif t == "list":
                 props[name] = parse_list(p)
     except (etree.XMLSyntaxError, IOError):
@@ -189,18 +206,23 @@ def write_config_file(filename=constants.CONFIG_FILE_LOCATION):
         e.set("type", pref.type)
         e.set("name", key)
         data = prefs[key] if key in prefs else pref.value
-        if pref.type in ["int", "bool"]:
+        if pref.type in ["int", "bool", "str"]:
             e.text = str(data)
         elif pref.type == "list":
-            for v in data:
-                f = etree.SubElement(e, "preference-item")
-                f.set("type", pref.itemtype)
-                if pref.itemtype == "file":
+            if pref.itemtype == "file":
+                for v in data:
+                    f = etree.SubElement(e, "preference-item")
+                    f.set("type", pref.itemtype)
                     for k0, v0 in v.items():
                         g = etree.SubElement(f, "preference-item")
                         g.set("name", k0)
                         g.set("type", v0["type"])
                         g.text = v0["value"]
+            elif pref.itemtype == "str":
+                for v in data:
+                    f = etree.SubElement(e, "preference-item")
+                    f.set("type", "str")
+                    f.text = v
     if not os.path.isdir(constants.APPLICATION_DIRECTORY):
         os.mkdir(constants.APPLICATION_DIRECTORY)
     contents = etree.tostring(root, xml_declaration=True, encoding="UTF-8")
