@@ -23,6 +23,11 @@ import pangocairo
 
 import constants
 import cPalabra
+from gui_common import (
+    create_label,
+    create_scroll,
+    create_drawing_area,
+)
 from preferences import read_pref_color
 
 SETTINGS_PREVIEW = {
@@ -208,7 +213,6 @@ class GridViewProperties:
             return sx
         elif y is not None:
             return sy
-        return None
         
     def screen_to_grid(self, screen_x, screen_y):
         """
@@ -602,13 +606,19 @@ class GridView:
         # borders with normal line width
         ctx_set_line_width(props_line_width)
         ctx_set_source_rgb(*[c / 65535.0 for c in props_border_color])
+        for rx, ry, rdx, rdy, bar, border, special in the_lines:
+            if not bar and border and special:
+                ctx_move_to(rx, ry)
+                ctx_rel_line_to(rdx, rdy)
+        ctx_stroke()
+        # lines that are sticking out with normal line width
+        ctx_set_source_rgb(*[c / 65535.0 for c in props_line_color])
         for rx, ry, rdx, rdy, bar, border, special in the_lines:
             if not bar and not border and special:
                 ctx_move_to(rx, ry)
                 ctx_rel_line_to(rdx, rdy)
         ctx_stroke()
         # lines
-        ctx_set_source_rgb(*[c / 65535.0 for c in props_line_color])
         for rx, ry, rdx, rdy, bar, border, special in the_lines:
             if not bar and not border and not special:
                 ctx_move_to(rx, ry)
@@ -659,20 +669,10 @@ class GridPreview(gtk.VBox):
         self.preview_surface = None
         self.preview_pattern = None
         self.render_mode = mode
-        
-        label = gtk.Label()
-        label.set_alignment(0, 0)
-        label.set_markup(''.join(["<b>", header, "</b>"]))
-        
-        self.drawing_area = gtk.DrawingArea()
-        self.drawing_area.connect("expose_event", self.on_expose_event)
-        
-        self.scrolled_window = gtk.ScrolledWindow(None, None)
-        self.scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        self.scrolled_window.add_with_viewport(self.drawing_area)
-        
-        self.pack_start(label, False, False, 6)
-        self.pack_start(self.scrolled_window, True, True, 0)
+        self.drawing_area = create_drawing_area(self.on_expose_event)
+        self.scrolled_window = create_scroll(self.drawing_area, viewport=True)
+        self.pack_start(create_label("<b>" + header + "</b>"), False, False, 6)
+        self.pack_start(self.scrolled_window)
         
     def display(self, grid):
         self.view = GridView(grid)
